#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : all_tasks
# @Time         : 2024/11/28 17:50
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 


# 所有异步任务
from meutils.pipe import *
from meutils.async_task import worker

for p in get_resolve_path('.', __file__).glob('*'):

    if (not p.name.startswith('_')) and p.name.endswith('.py'):
        logger.debug(p)
        _ = try_import(f"meutils.async_task.tasks.{p.stem}")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : html2image
# @Time         : 2024/9/18 14:58
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from html2image import Html2Image

hti = Html2Image()

html_content = Path('x.html').read_text()

# hti.screenshot(html_str=html_content, save_as='output_desktop.png', size=(1920, 1080))
hti.screenshot(html_str=html_content, save_as='output_mobile.png', size=(375, 812))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : token
# @Time         : 2024/7/19 13:56
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import os

from meutils.pipe import *

import requests
import json

# check token: todo todo todo
url = "https://api.chatfire.cn/v1/dashboard/billing/subscription"

# 创建
url = "https://api.chatfire.cn/api/token/"

payload = json.dumps({
    "name": "xxx",
    "remain_quota": 500000,
    "expired_time": -1,
    "unlimited_quota": False,
    "model_limits_enabled": False,
    "model_limits": ""
})
headers = {
}

response = requests.request("POST", url, headers=headers, data=payload)

print(response.text)

# 根据 key id 查询
import http.client

conn = http.client.HTTPSConnection("api.chatfire.cn")
payload = ''
conn.request("GET", "/api/token/1096", payload, headers)
res = conn.getresponse()
data = res.read()
print(data.decode("utf-8"))

# 修改key信息

import requests
import json

url = "https://api.chatfire.cn/api/token/"

payload = json.dumps({
    "id": 1096,
    "user_id": 1,
    "key": "SNzQfo2enYpVqWV08f2e9023170245878d58F92bD820Af814",
    "status": 1,
    "name": "xxx",
    "created_time": 1721368510,
    "accessed_time": 1721368510,
    "expired_time": -1,
    "remain_quota": 5000001,
    "unlimited_quota": False,
    "model_limits_enabled": False,
    "model_limits": "",
    "used_quota": 0,
    "DeletedAt": None
})

response = requests.request("PUT", url, headers=headers, data=payload)

print(response.text)

# 查key 余额与
import requests

url = "https://api.chatfire.cn/v1/dashboard/billing/usage" # ?start_date=2024-4-10&end_date=2024-7-19

payload = {}
headers = {
    'authorization': f'Bearer {os.getenv("OPENAI_API_KEY")}',
}

response = requests.request("GET", url, headers=headers, data=payload)

print(response.text)

# 查key 消费行为；可定位到 key id  user id
# https://api.chatfire.cn/api/log/token?key=sk-


# key信息
import requests

url = "https://api.chatfire.cn/v1/dashboard/billing/subscription"

payload = {}
headers = {
    'authorization': f'Bearer {os.getenv("OPENAI_API_KEY")}',
    'priority': 'u=1, i',
    'User-Agent': 'Apifox/1.0.0 (https://apifox.com)'
}

response = requests.request("GET", url, headers=headers, data=payload)

print(response.text)

# 查询用户下所有 key
# https://api.chatfire.cn/api/token/?p=0&size=1
# https://api.chatfire.cn/api/token/1096 # 根据超级管理员查询用户access token然后再查询key信息



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : audio
# @Time         : 2025/7/17 13:13
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 


from openai import AsyncOpenAI

from meutils.pipe import *
from meutils.io.files_utils import to_url, to_bytes
from meutils.llm.clients import AsyncOpenAI
from meutils.llm.openai_utils import to_openai_params
from meutils.llm.check_utils import get_valid_token_for_fal

from meutils.config_utils.lark_utils import get_next_token_for_polling, get_next_token
from meutils.schemas.openai_types import STTRequest, TTSRequest

FEISHU_URL = "https://xchatllm.feishu.cn/sheets/Z59Js10DbhT8wdt72LachSDlnlf?sheet=iFRwmM"  # 异步任务号池

BASE_URL = "https://ai.gitee.com/v1"

from fal_client.client import AsyncClient, SyncClient, Status, FalClientError


# try:
#
#     data = await AsyncClient(key=token).run(
#         application=request.model,
#         arguments=arguments,
#     )
#     logger.debug(data)
#     return ImagesResponse(data=data.get("images"), timings={"inference": time.time() - s})
#
# except Exception as exc:  #
#     logger.error(exc)
#     from fastapi import HTTPException, status
#
#     raise HTTPException(
#         status_code=500,
#         detail=f"Failed to generate image: {exc}",
#     )

# "fal-ai/minimax/speech-02-turbo"
async def text_to_speech(request: TTSRequest, api_key: Optional[str] = None):
    if isinstance(api_key, str) and api_key.startswith("oneapi:"):
        api_key = api_key.removeprefix("oneapi:")

    api_key = api_key or await get_valid_token_for_fal()

    payload = request.model_dump(exclude_none=True)
    payload = {
        "text": request.input,
        "stream": False,
        "output_format": request.response_format if request.response_format in {"url", "hex"} else "hex",
        "voice_setting":
            {
                "speed": request.speed or 1,
                "voice_id": request.voice or "wumei_yujie",
                "emotion": "happy"
            },
        **payload
    }

    try:

        data = await AsyncClient(key=api_key).run(
            application=request.model,
            arguments=payload,
        )
        # {'audio': {'url': 'https://v3.fal.media/files/zebra/wTM2HIdYkQTl0q5qdTDl9_speech.mp3', 'content_type': 'audio/mpeg', 'file_name': 'speech.mp3', 'file_size': 64034}, 'duration_ms': 3888}
        logger.debug(data)
        logger.debug(request)

        if request.response_format not in {"url", "hex"}:  ########### 没传进去
            data = await to_bytes(data["audio"]["url"])

        return data


    # """
    # 2025-12-10 22:12:56.768 | DEBUG | meutils.apis.fal.audio:text_to_speech:76 - {'audio': {'url': 'https://v3b.fal.media/files/b/0a85bffd/LRjnbrbeuESwx4hoZfIaD_speech.mp3', 'content_type': 'audio/mpeg', 'file_name': 'speech.mp3', 'file_size': 83061}, 'duration_ms': 5076}
    #
    # """

    except Exception as exc:  #
        logger.error(exc)
        from fastapi import HTTPException, status

        raise HTTPException(
            status_code=500,
            detail=f"Failed: {exc}",
        )


if __name__ == '__main__':
    data = {
        "model": "fal-ai/minimax/speech-02-turbo",
        "input": "根据 prompt audio url克隆音色",
        "response_format": "url"

        # "voice": "Wise_Woman"
    }

    request = TTSRequest(**data)
    print(request)

    arun(text_to_speech(request))
    # print(MODELS.values())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : pd
# @Time         : 2025/2/19 13:23
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

df = pd.read_csv('x.txt', sep='--', header=None)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : common
# @Time         : 2025/3/17 18:44
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from httpx import AsyncClient

from meutils.pipe import *
from meutils.decorators.retry import retrying
from meutils.caches import cache, rcache


@retrying()
@rcache(ttl=600)
async def _url_reader(url: str):
    """markdown"""

    async with AsyncClient(base_url="https://r.jina.ai", headers={}, timeout=300) as client:
        response = await client.get(f"/{url}")
        response.raise_for_status()

        return response.text


async def url_reader(urls: Union[str, List[str]]):
    if isinstance(urls, str):
        return await _url_reader(urls)

    tasks = [_url_reader(url) for url in urls]
    return await asyncio.gather(*tasks)


if __name__ == '__main__':
    url = "https://top.baidu.com/board?tab=realtime"
    url = "https://mp.weixin.qq.com/s/qvR1KbYVOq7XQO1tmEMVJA"

    # print(arun(url_reader(url)))

    print(arun(url_reader([url])))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : videos
# @Time         : 2025/12/22 14:56
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 


from meutils.pipe import *
from meutils.schemas.video_types import SoraVideoRequest, Video

import replicate


class Tasks(object):

    def __init__(self, base_url: Optional[str] = None, api_key: str = None):
        api_key = api_key or os.getenv("REPLICATE_API_KEY")
        self.client = replicate.client.Client(api_token=api_key)

    async def create(self, request: SoraVideoRequest):

        payload = {
            "prompt": request.prompt,
            "duration": int(request.seconds),
            "resolution": request.resolution,
            "multi_shots": False,
            "negative_prompt": "",
            "enable_prompt_expansion": True,
            # "image": "https://replicate.delivery/pbxt/OF1th8iUuEue0j7p1dces9rrhbss2tri6zIrvWxFSUEAaiVw/replicate-prediction-gbdjrctjksrme0cv4m58vwtdtr.jpg",

        }
        if request.size:
            # if "x" not in request.size:
            #     raise ValueError(f"size must be in format of 1280x720")
            w, h = 16, 9
            if 'x' in request.size:
                w, h = map(int, request.size.split('x'))
            elif ':' in request.size:
                w, h = map(int, request.size.split(':'))
            elif '*' in request.size:
                w, h = map(int, request.size.split('*'))

            if request.resolution in {"720p", None}:
                payload["size"] = "1280*720" if w > h else "720*1280"
            elif request.resolution == "1080p":
                payload["size"] = "1920*1080" if w > h else "1080*1920"

        if request.input_reference:
            request.model = request.model.replace("t2v", "i2v")
            payload["image"] = request.input_reference[0]

        if request.audio:
            payload["audio"] = request.audio

        logany(request)
        logany(bjson(payload))

        response = await self.client.predictions.async_create(
            model=request.model,
            input=payload
        )
        """
        Prediction(id='vzykqfxg2hrmt0cv8khte3g11g', model='wan-video/wan-2.6-i2v', version='hidden', status='starting', input={'duration': 5, 'enable_prompt_expansion': True, 'image': 'https://replicate.delivery/pbxt/OF1th8iUuEue0j7p1dces9rrhbss2tri6zIrvWxFSUEAaiVw/replicate-prediction-gbdjrctjksrme0cv4m58vwtdtr.jpg', 'multi_shots': False, 'negative_prompt': '', 'prompt': 'The vintage clock on the table starts ticking, gears visibly turning inside the glass case, pendulum swinging smoothly, dust particles floating in sunlight beams, close-up macro shot with shallow depth of field.', 'resolution': '720p'}, output=None, logs='', error=None, metrics=None, created_at='2025-12-22T07:08:51.092Z', started_at=None, completed_at=None, urls={'cancel': 'https://api.replicate.com/v1/predictions/vzykqfxg2hrmt0cv8khte3g11g/cancel', 'get': 'https://api.replicate.com/v1/predictions/vzykqfxg2hrmt0cv8khte3g11g', 'stream': 'https://stream.replicate.com/v1/files/jbxs-v3vlgpd4avh5fyfsjinxacrsxb2uj242muyvn4k6jabuljjjfwrq', 'web': 'https://replicate.com/p/vzykqfxg2hrmt0cv8khte3g11g'})

        """

        return response.dict()  # vzykqfxg2hrmt0cv8khte3g11g

    async def get(self, task_id: str):
        response = await self.client.predictions.async_get(task_id)
        #
        logger.debug(bjson(response))

        response = response.dict()
        #
        video = Video(
            id=task_id,
            status=response,

            model=response.get("model"),
            video_url=response.get("output"),

            error=response.get("error")
        )

        # logger.debug(bjson(video))

        return video


if __name__ == '__main__':
    model = "wan-video/wan-2.6-i2v"

    request = SoraVideoRequest(
        model=model,
        # prompt="Put the woman next to the house",
        # prompt="一个裸体女人",
        # prompt='带个墨镜',
        # prompt=prompt,
        # size="2048x2048",

        # aspect_ratio="match_input_image",
        # input_image_1="https://replicate.delivery/pbxt/N7gRAUNcVF6HarL0hdAQA2JYNMlJD52LP1wyaIWRUXWeHzqT/0_1-1.webp",
        # input_image_2="https://replicate.delivery/pbxt/N7gRAK5kbPwdsbOpqgyAIOFQX45U6suTlbL6ws2N74SnGFpo/test.jpg",
        # image="https://s3.ffire.cc/files/jimeng.jpg",
        # image="https://replicate.delivery/pbxt/N7gRAUNcVF6HarL0hdAQA2JYNMlJD52LP1wyaIWRUXWeHzqT/0_1-1.webp"
    )
    print(request)

    # arun(Tasks().create(request))

    task_id = "vzykqfxg2hrmt0cv8khte3g11g"
    arun(Tasks().get(task_id))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : crontab
# @Time         : 2021/2/7 5:33 下午
# @Author       : yuanjie
# @WeChat       : 313303303
# @Software     : PyCharm
# @Description  :
# @Warning      : job.slices = ''  # 不再重复添加时间 * * * * *

from meutils.pipe import *
from meutils.path_utils import file2json

class Cron(object):
    """
    Crontab 增删改查
    """

    def __init__(self, **kwargs):
        pass

    def add(self, command, comment):
        with CronTab(True) as cron:
            logger.warning(f"ADD cron: {command}")
            job = cron.new(command, comment, pre_comment=True)
            job.slices = ''  # 不再重复添加时间 * * * * *

    def remove(self, comment):
        """

        :param comment: 仅支持comment剔除
        :return:
        """
        with CronTab(True) as cron:
            jobs = cron.find_comment(comment)
            for job in jobs:
                cron.remove(job)

    def update_from_file(self, path):
        """从zk同步cron: 可配置每天最后一分钟同步crontab配置信息（zk/yaml） 59 23 * * *

        mecli-cron - update_from_file

        :param path: zk/yaml /push/crontab/mitv
            comment1:
              - cmd1
              - cmd2
        :return:
        """
        if Path(path).is_file():
            crontabs = file2json(path)
        else:
            from meutils.zk_utils import get_zk_config
            crontabs = get_zk_config(path)

        logger.info(f"Crontab update to: {bjson(crontabs)}")

        # todo: 统一管理统一前缀的， filter(lambda prefix: str.startswith(prefix), _comments)
        with CronTab(True) as cron:
            for comment, cmds in crontabs.items():
                # 删除
                jobs = cron.find_comment(comment)
                for job in jobs:
                    cron.remove(job)

                # 新增
                for cmd in cmds:
                    cmd = cmd.strip().split() | xjoin
                    job = cron.new(cmd, comment, pre_comment=True)  # cmd里特殊字符处理
                    job.slices = ''  # 不再重复添加时间 * * * * *


def main():
    fire.Fire(Cron)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : seconds
# @Time         : 2025/10/31 19:15
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : todo 接口
import math

import numpy as np

from meutils.pipe import *

TIMES = 4
TIMES_5 = 0.5
TIMES_6 = 0.6
TIMES_7 = 0.7
TIMES_8 = 0.8

data = {
    # kling
    "kling-video-o1": 0.4,
    "kling-video-o1-edit": 0.4,
    "kling-video-2.6": 0.07 * TIMES,
    "klingai/avatar-standard": 0.059 * TIMES,
    "klingai/avatar-pro": 0.121 * TIMES,
    "bytedance/omnihuman/v1.5": 0.168 * TIMES,

    # hailuo
    "minimax-hailuo-02_768p": 0.56 / 10 * TIMES,
    "minimax-hailuo-02_1080p": 0.49 / 6 * TIMES,
    "minimax-hailuo-2.3_768p": 0.56 / 10 * TIMES,
    "minimax-hailuo-2.3_1080p": 0.49 / 6 * TIMES,
    "minimax-hailuo-2.3-fast_768p": 0.32 / 10 * TIMES,
    "minimax-hailuo-2.3-fast_1080p": 0.33 / 6 * TIMES,

    "viduq2-turbo_720p": 0.2,
    "viduq2-turbo_1080p": 0.8,
    "viduq2-pro_720p": 0.2,
    "viduq2-pro_1080p": 0.8,

    # wan
    "wan2.5_480p": 0.05 * TIMES,
    "wan2.5_720p": 0.1 * TIMES,
    "wan2.5_1080p": 0.15 * TIMES,
    "wan2.6_720p": 0.1 * TIMES,
    "wan2.6_1080p": 0.15 * TIMES,

    # sora
    "sora-2": 0.1 * TIMES,
    "sora-2-pro": 0.3 * TIMES,

    # veo
    "veo3.1_720p": 0.4 * TIMES,
    "veo3.1_1080p": 0.4 * TIMES,
    "veo3.1-fast_720p": 0.15 * TIMES,
    "veo3.1-fast_1080p": 0.15 * TIMES,

    # seedance
    'doubao-seedance-1-0-pro_480p': 1.46 / 10 * 0.6,
    'doubao-seedance-1-0-pro_720p': 3.38 / 10 * 0.6,
    'doubao-seedance-1-0-pro_1080p': 7.34 / 10 * 0.6,
    'doubao-seedance-1-0-pro-fast_480p': 0.41 / 10 * 0.9,
    'doubao-seedance-1-0-pro-fast_720p': 0.95 / 10 * 0.9,
    'doubao-seedance-1-0-pro-fast_1080p': 2.06 / 10 * 0.9,
    'doubao-seedance-1-0-lite_480p': 0.41 / 10 * 0.9,
    'doubao-seedance-1-0-lite_720p': 0.95 / 10 * 0.9,
    'doubao-seedance-1-0-lite_1080p': 2.06 / 10 * 0.9,

    "doubao-seedance-1-5-pro_480p": 0.8 / 10 * 0.6,
    "doubao-seedance-1-5-pro_720p": 1.6 / 10 * 0.6,
    "doubao-seedance-1-5-pro_1080p": 3 / 10 * 0.6,

    # 'doubao-seedance-1-0-lite_480p': 0.97 / 10,
    # 'doubao-seedance-1-0-lite_720p': 2.26 / 10,
    # 'doubao-seedance-1-0-lite_1080p': 4.9 / 10,

    # 混元
    "hunyuan-video-v1.5": 0.2,

    # pixverse
    "pixverse-v5.5_720p": 1.386 / 10 * TIMES,
    "pixverse-v5.5_1080p": 2.52 / 8 * TIMES,

    # runway
    "runway/gen4-turbo": 0.053 * TIMES,
    "runway/gen4-aleph": 0.158 * TIMES,

    # luma
    "luma/ray-flash-2_720p": 0.25 / 5 * TIMES,
    "luma/ray-flash-2_1080p": 0.3 / 5 * TIMES,
    "luma/ray-flash-2_4k": 0.5 / 5 * TIMES,

    "luma/ray-2_720p": 0.25 / 4 * TIMES,
    "luma/ray-2_1080p": 0.3 / 4 * TIMES,
    "luma/ray-2_4k": 0.5 / 4 * TIMES,

}

if __name__ == '__main__':
    _ = ','.join([k for k, v in data.items() if k.startswith("doubao")])
    print(_)

    data = dict(zip(data, np.ceil(1000 * np.array(list(data.values()))) / 1000))
    print(json.dumps(data, indent=4))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : scenedetect_demo
# @Time         : 2023/11/17 16:07
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

import scenedetect
from scenedetect.scene_manager import SceneManager, save_images as _save_images
from scenedetect.scene_manager import write_scene_list as _write_scene_list
from scenedetect.scene_manager import write_scene_list_html as _write_scene_list_html

from scenedetect.stats_manager import StatsManager
from scenedetect.detectors.content_detector import ContentDetector, SceneDetector

import pysrt
from pysrt.srttime import SubRipTime


class SceneDetect(object):

    def __init__(self, video_path, detector: Optional[SceneDetector] = None):
        self.videostream = scenedetect.open_video(video_path)
        self.scene_manager = SceneManager()
        self.scene_manager.add_detector(detector or ContentDetector())
        self.scene_manager.detect_scenes(self.videostream, show_progress=True)

    def save_images(
            self,
            num_images=1,
            image_name_template: str = '$VIDEO_NAME-Scene-$SCENE_NUMBER-$IMAGE_NUMBER',
            output_dir: Optional[str] = None,
            show_progress: Optional[bool] = True,
            **kwargs
    ):
        kwargs['num_images'] = num_images
        kwargs['image_name_template'] = image_name_template
        kwargs['output_dir'] = output_dir
        kwargs['show_progress'] = show_progress

        image_filenames = _save_images(self.scene_list, self.videostream, **kwargs)

        return image_filenames

    def to_srt(self, texts: Optional[List[str]] = None, filename="example.srt"):
        """
            text为空会临时放入time_delta
        :param texts:
        :param filename:
        :return:
        """
        # 创建一个 SubRipFile 实例
        texts = texts or [""] * len(self.scene_list)

        subs = pysrt.SubRipFile()
        for index, ((scene_s, scene_e), text) in enumerate(zip(self.scene_list, texts)):
            time_delta = scene_e.get_seconds() - scene_s.get_seconds()
            text = text or int(time_delta * 1000)
            start = SubRipTime(*re.split('[:.]', scene_s.get_timecode()) | xmap(float))
            end = SubRipTime(*re.split('[:.]', scene_e.get_timecode()) | xmap(float))
            # 创建一个新的字幕项
            sub = pysrt.SubRipItem(index=index, start=start, end=end, text=text)
            # position 属性允许你指定字幕在视频画面中的位置。这个属性是一个元组，包含两个整数值，可以临时放一下时长
            subs.append(sub)
        # 保存字幕文件
        subs.save(filename, encoding='utf-8')

    @cached_property
    def scene_list(self):
        return self.scene_manager.get_scene_list()

    def write_scene_list_html(self, output_html_filename, **kwargs):
        _write_scene_list_html(output_html_filename, self.scene_list, **kwargs)

    def write_scene_list(self, output_csv_filename, **kwargs):
        with open(output_csv_filename, 'w') as f:
            _write_scene_list(f, self.scene_list, **kwargs)


if __name__ == '__main__':
    p = "/Users/betterme/PycharmProjects/AI/MeUtils/meutils/ai_video/312_1700705412.mp4"

    s = SceneDetect(p)
    # image_filenames = s.save_images(output_dir='xx')
    # print(image_filenames)
    # print(s.scene_list)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : __init__.py
# @Time         : 2023/11/21 18:44
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

from meutils.ai_audio.utils import to_audio

for p in Path('.').glob('*.mp3'):
    # to_audio(p, p.name.replace('mp3', 'pcm'), format="s16le", codec="pcm_s16le")
    to_audio(open("/Users/betterme/PycharmProjects/AI/MeUtils/meutils/ai_audio/asr/2022112519张健涛29.mp3", 'wb'), p, format="mp3")

# from pydub import AudioSegment
#
# # 读取 MP3 文件
# audio = AudioSegment.from_file("/Users/betterme/PycharmProjects/AI/MeUtils/meutils/ai_audio/asr/音频文件夹/20190101-section_2.mp3", format="mp3")
#
# # 将 MP3 文件转换为 PCM 格式
# pcm_data = audio.raw_data
#
# # 将 PCM 数据保存到文件
# with open("output.pcm", "wb") as f:
#     f.write(pcm_data)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : search
# @Time         : 2025/1/27 13:41
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : todo: 格式匹配
# https://github.com/deepseek-ai/DeepSeek-R1

from meutils.pipe import *
from meutils.llm.clients import AsyncOpenAI, chatfire_client, zhipuai_client, moonshot_client
from meutils.llm.openai_utils import to_openai_params
from meutils.schemas.openai_types import chat_completion, chat_completion_chunk, ChatCompletionRequest, CompletionUsage


class Completions(object):

    def __init__(self, api_key: Optional[str] = None):
        self.api_key = api_key

    async def create(self, request: ChatCompletionRequest):
        request.model = request.model.removesuffix("-search")

        if request.model.startswith("baichuan"):
            base_url = os.getenv("BAICHUAN_BASE_URL")
            api_key = self.api_key or os.getenv("BAICHUAN_API_KEY")

            request.tools = [
                {
                    "type": "web_search",
                    "web_search": {
                        "enable": True,
                        "search_mode": "performance_first"
                    }
                }
            ]
            data = to_openai_params(request)
            client = AsyncOpenAI(base_url=base_url, api_key=api_key)
            completion = await client.chat.completions.create(**data)
            return completion

        elif request.model.startswith(("moonshot", "kimi")):
            tool_call_name = "$web_search"
            request.tools = [
                {
                    "type": "builtin_function",  # <-- 我们使用 builtin_function 来表示 Kimi 内置工具，也用于区分普通 function
                    "function": {
                        "name": "$web_search",
                    },
                },
            ]

            data = to_openai_params(request)
            completion = await moonshot_client.chat.completions.create(**data)

            tool_call = completion.choices[0].message.tool_calls[0]
            tool_call_arguments = tool_call.function.arguments
            print(tool_call_arguments)

            request.messages.append({
                "role": "tool",
                "tool_call_id": tool_call.id,
                "name": tool_call_name,
                "content": tool_call_arguments,
            })
            logger.debug(request.messages)
            data = to_openai_params(request)
            logger.debug(data)
            completion = await moonshot_client.chat.completions.create(**data)
            return completion

        elif request.model.startswith("doubao"):
            base_url = os.getenv("DOUBAO_BASE_URL")
            api_key = self.api_key or os.getenv("DOUBAO_API_KEY")

            request.model = "bot-20250127143547-c8q8m"
            request.tools = [
                {
                    "type": "web_search",
                    "web_search": {
                        "enable": True,
                        "search_mode": "performance_first"
                    }
                }
            ]
            data = to_openai_params(request)
            client = AsyncOpenAI(base_url=base_url, api_key=api_key)

            completion = await client.chat.completions.create(**data)
            return completion

        elif request.model.startswith("search"):  # 智谱
            data = to_openai_params(request)

            search_completion = await zhipuai_client.chat.completions.create(**data)
            search_completion.model_dump_json(indent=4)


        else:
            # 搜索
            data = to_openai_params(request)
            data['model'] = "web-search-pro"
            data['stream'] = False
            search_completion = await zhipuai_client.chat.completions.create(**data)
            logger.debug(search_completion.model_dump_json(indent=4))  # todo: 返回详细信息
            # todo: chat搜索接口 搜索内容重排序 搜索聚合

            # 大模型
            request.messages.append(
                {
                    "role": "user",
                    "content": search_completion.model_dump_json(indent=4),
                }
            )

            data = to_openai_params(request)
            completion = await chatfire_client.chat.completions.create(**data)
            return completion


if __name__ == '__main__':
    request = ChatCompletionRequest(
        # model="baichuan4-turbo",
        # model="xx",
        # model="deepseek-r1",
        # model="deepseek-r1:1.5b",
        model="deepseek-r1:32b",

        # model="moonshot-v1-8k",
        # model="doubao",

        messages=[
            {"role": "user", "content": "《哪吒之魔童闹海》现在的票房是多少"}
        ],

        stream=False
    )

    arun(Completions().create(request))

    # async def test():
    #     for i in await Completions().create(request):
    #         print(i)
    #
    #
    # arun(test())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : monitor
# @Time         : 2022/7/13 下午1:28
# @Author       : yuanjie
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

import typer
from meutils.pipe import *
from meutils.tools import service_monitor as _service_monitor


@cli.command()
def service_monitor(filename):
    """传入配置文件"""
    _ = _service_monitor.main(filename)

    typer.echo(_)


@cli.command()
def xx(filename):
    """传入配置文件"""
    return _service_monitor.main(filename)


if __name__ == '__main__':
    cli()



import os
from meutils.pipe import *

from pathlib import Path
from pydantic import BaseModel
from typing import *
import websocket
import datetime
import hashlib
import base64
import hmac
import json
from urllib.parse import urlencode
import time
import ssl
from wsgiref.handlers import format_date_time
from datetime import datetime
from time import mktime
import _thread as thread
import ffmpy

STATUS_FIRST_FRAME = 0  # 第一帧的标识
STATUS_CONTINUE_FRAME = 1  # 中间帧标识
STATUS_LAST_FRAME = 2  # 最后一帧的标识


class Word(BaseModel):
    sc: int
    w: str


class Words(BaseModel):
    bg: int
    cw: List[Word]


class Result(BaseModel):
    ws: List[Words]


class Ws_Param(object):
    # 初始化
    def __init__(self, APPID, APIKey, APISecret, AudioFile):
        self.APPID = APPID
        self.APIKey = APIKey
        self.APISecret = APISecret
        self.AudioFile = AudioFile

        # 公共参数(common)
        self.CommonArgs = {"app_id": self.APPID}
        # 业务参数(business)，更多个性化参数可在官网查看
        self.BusinessArgs = {"domain": "iat", "language": "zh_cn", "accent": "mandarin", "vinfo": 1, "vad_eos": 10000}

    # 生成url
    def create_url(self):
        url = 'wss://ws-api.xfyun.cn/v2/iat'
        # 生成RFC1123格式的时间戳
        now = datetime.now()
        date = format_date_time(mktime(now.timetuple()))

        # 拼接字符串
        signature_origin = "host: " + "ws-api.xfyun.cn" + "\n"
        signature_origin += "date: " + date + "\n"
        signature_origin += "GET " + "/v2/iat " + "HTTP/1.1"
        # 进行hmac-sha256进行加密
        signature_sha = hmac.new(self.APISecret.encode('utf-8'), signature_origin.encode('utf-8'),
                                 digestmod=hashlib.sha256).digest()
        signature_sha = base64.b64encode(signature_sha).decode(encoding='utf-8')

        authorization_origin = "api_key=\"%s\", algorithm=\"%s\", headers=\"%s\", signature=\"%s\"" % (
            self.APIKey, "hmac-sha256", "host date request-line", signature_sha)
        authorization = base64.b64encode(authorization_origin.encode('utf-8')).decode(encoding='utf-8')
        # 将请求的鉴权参数组合为字典
        v = {
            "authorization": authorization,
            "date": date,
            "host": "ws-api.xfyun.cn"
        }
        # 拼接鉴权参数，生成url
        url = url + '?' + urlencode(v)
        # print("date: ",date)
        # print("v: ",v)
        # 此处打印出建立连接时候的url,参考本demo的时候可取消上方打印的注释，比对相同参数时生成的url与自己代码生成的url是否一致
        # print('websocket url :', url)
        return url


# 收到websocket消息的处理
def on_message(ws, message):
    try:
        code = json.loads(message)["code"]
        sid = json.loads(message)["sid"]
        if code != 0:
            errMsg = json.loads(message)["message"]
            print("sid:%s call error:%s code is:%s" % (sid, errMsg, code))

        else:
            result = json.loads(message)["data"]["result"]
            rst = Result(**result)
            text = ''.join(w.cw[0].w for w in rst.ws)

            rst = {'text': text, 'begin': rst.ws[0].bg, 'end': rst.ws[-1].bg}

            with open('asr.json', 'a') as file:
                line = f"{json.dumps(rst, ensure_ascii=False)}\n"
                file.write(line)

    except Exception as e:
        print("receive msg,but parse exception:", e)


# 收到websocket错误的处理
def on_error(ws, error):
    print("### error:", error)


# 收到websocket关闭的处理
def on_close(ws):
    print("### closed ###")


# 收到websocket连接建立的处理
def on_open(ws):
    def run(*args):
        frameSize = 8000  # 每一帧的音频大小
        intervel = 0.04  # 发送音频间隔(单位:s)
        status = STATUS_FIRST_FRAME  # 音频的状态信息，标识音频是第一帧，还是中间帧、最后一帧

        with open(wsParam.AudioFile, "rb") as fp:
            while True:
                buf = fp.read(frameSize)
                # 文件结束
                if not buf:
                    status = STATUS_LAST_FRAME
                # 第一帧处理
                # 发送第一帧音频，带business 参数
                # appid 必须带上，只需第一帧发送
                if status == STATUS_FIRST_FRAME:

                    d = {"common": wsParam.CommonArgs,
                         "business": wsParam.BusinessArgs,
                         "data": {"status": 0, "format": "audio/L16;rate=16000",
                                  "audio": str(base64.b64encode(buf), 'utf-8'),
                                  "encoding": "raw"}}
                    d = json.dumps(d)
                    ws.send(d)
                    status = STATUS_CONTINUE_FRAME
                # 中间帧处理
                elif status == STATUS_CONTINUE_FRAME:
                    d = {"data": {"status": 1, "format": "audio/L16;rate=16000",
                                  "audio": str(base64.b64encode(buf), 'utf-8'),
                                  "encoding": "raw"}}
                    ws.send(json.dumps(d))
                # 最后一帧处理
                elif status == STATUS_LAST_FRAME:
                    d = {"data": {"status": 2, "format": "audio/L16;rate=16000",
                                  "audio": str(base64.b64encode(buf), 'utf-8'),
                                  "encoding": "raw"}}
                    ws.send(json.dumps(d))
                    time.sleep(1)
                    break
                # 模拟音频采样间隔
                time.sleep(intervel)
        ws.close()

    thread.start_new_thread(run, ())


def sound2text(filename: str):
    # 测试时候在此处正确填写相关信息即可运行
    time1 = datetime.now()

    pcm_file = Path(filename).name.replace('mp3', 'pcm').replace('wav', 'pcm')
    # 将mp3转化没pcm
    ff = ffmpy.FFmpeg(
        # executable='D:\\ffmpeg\\ffmpeg.exe',
        inputs={filename: "-y"}, outputs={pcm_file: "-acodec pcm_s16le -f s16le -ac 1 -ar 16000"}
    )

    # print(ff.cmd)
    ff.run()
    global wsParam
    appid, apikey, apisecret = os.getenv('XUNFEI_API_KEY').split('|')
    # wsParam = Ws_Param(APPID='fd214691', APISecret='NmE0OTlmNDFiNjA2YzExZmJmMTAzY2U3',
    #                    APIKey='b4a289cffdc181fad445418cc61fd40a',
    #                    AudioFile=r'trial.pcm')

    wsParam = Ws_Param(APPID=appid, APISecret=apisecret, APIKey=apikey, AudioFile=pcm_file)

    websocket.enableTrace(False)
    wsUrl = wsParam.create_url()
    ws = websocket.WebSocketApp(wsUrl, on_message=on_message, on_error=on_error, on_close=on_close)
    ws.on_open = on_open
    ws.run_forever(sslopt={"cert_reqs": ssl.CERT_NONE})
    time2 = datetime.now()
    print(time2 - time1)


if __name__ == "__main__":
    # 企微有时候看不到，售后微信：313303303
    # os.environ['XUNFEI_API_KEY'] = "appid|apikey|apisecret"  # 填写你自己的key

    p = '/Users/betterme/PycharmProjects/AI/zh_.wav'
    # p = "/Users/betterme/PycharmProjects/AI/MeUtils/meutils/ai_audio/asr/音频文件夹/20190101-section_2.mp3"

    sound2text(p)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : np_utils
# @Time         : 2020/11/12 11:35 上午
# @Author       : yuanjie
# @Email        : meutils@qq.com
# @Software     : PyCharm
# @Description  : 

from itertools import combinations
from collections import defaultdict

import numpy as np

# ME
from meutils.pipe import *

# 分组
# np.array_split(range(6), 3)
# iteration_utilities.split
# iteration_utilities.grouper([1,2,3,4], 2) | xlist


# 展平
"""
l=[[1,2,3],[4,[5],[6,7]],[8,[9,[10]]]]*1000
from iteration_utilities import deepflatten
_ = list(deepflatten(l)) # 快十倍
_ = sum(l, [])
"""


def normalize(x: np.ndarray) -> np.ndarray:
    """clip
        x /= np.clip(np.linalg.norm(x, axis=-1, keepdims=True), 1e-12, None)
    """
    return x / np.linalg.norm(x, axis=-1, keepdims=True)


def cosine(v1, v2):  # 相似度不是距离
    """
    v1 = np.array([[1, 2], [3, 4]])
    v2 = np.array([[5, 6], [7, 8], [5, 6]])
    cosine_dist(v1, v2)
    """
    v1, v2 = map(np.array, (v1, v2))

    assert v1.ndim == v2.ndim
    if v1.ndim == 1:
        v1, v2 = v1.reshape(1, -1), v2.reshape(1, -1)

    from sklearn.metrics.pairwise import cosine_similarity
    return cosine_similarity(v1, v2).clip(0, 1)


def cosine_topk(v1, v2, topk=10):  # 相似度不是距离
    dist = - cosine(v1, v2)
    idxs = np.argsort(dist)[:, :topk]
    scores = - np.take_along_axis(dist, idxs, -1)  # 取出得分
    return list(zip(idxs, scores))[0] #########
    # return list(zip(idxs.tolist(), scores.tolist()))



def cosine_similarity(v1, v2):
    """先归一化再点乘，比from sklearn.metrics.pairwise import cosine_similarity 快10倍"""
    v1, v2 = map(lambda v: v / np.linalg.norm(np.atleast_2d(v), axis=1, keepdims=True), [v1, v2])
    return v1 @ v2.T


def similarity_search_by_vector(v1, v2, topk=10):
    """

    :param v1:
    :param v2:
    :param topk:
    :return: idxs, scores
    """
    dist = - cosine_similarity(v1, v2)
    idxs = np.argsort(dist)[:, :topk]
    scores = - np.take_along_axis(dist, idxs, -1)  # 取出得分
    return idxs, scores


def cooccurrence_matrix(texts, window_size=2):
    """
    构建共现矩阵
    :param texts: 文本列表
    :param window_size: 单词之间的最大距离
    :return: 共现矩阵

        data_list = [
                ['I' ,'like','learning', 'like'],
                ['I' ,'like','playing'],
            ]
        print(cooccurrence_matrix(data_list, 4))
    """
    # 统计单词出现的次数
    word_counts = defaultdict(int)
    for text in texts:
        for word in text:
            word_counts[word] += 1

    # 创建单词-id映射
    word_to_id = {word: i for i, word in enumerate(word_counts)}
    i2w = {i: w for w, i in word_to_id.items()}

    # 初始化共现矩阵
    matrix = np.zeros((len(word_counts), len(word_counts)))

    # 统计共现次数
    for text in tqdm(texts):
        for i, j in combinations(range(len(text)), 2):
            if abs(i - j) <= window_size:
                word_i, word_j = text[i], text[j]
                if word_i in word_to_id and word_j in word_to_id:
                    matrix[word_to_id[word_i], word_to_id[word_j]] += 1
                    matrix[word_to_id[word_j], word_to_id[word_i]] += 1
    index = list(i2w.values())
    columns = list(i2w.values())
    return pd.DataFrame(matrix, index, columns)


if __name__ == "__main__":
    import time
    from sklearn.metrics.pairwise import cosine_similarity

    x = np.random.rand(10, 128)
    y = np.random.rand(1000000, 128)

    print(cosine_topk(x[:1], x))
    idxs, scores = cosine_topk(x[:1], x)
    print(idxs)

    print(similarity_search_by_vector(x, y, 3)[0])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : lip_sync
# @Time         : 2025/1/3 16:17
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
"""
1. 上传图片 image_to_avatar检测
2. 上传视频 video_to_avatar检测
3. 上传音频+创建任务

"""
import asyncio

from meutils.pipe import *
from meutils.str_utils.json_utils import json_path

from meutils.schemas.jimeng_types import BASE_URL
from meutils.schemas.video_types import VideoRequest
from meutils.schemas.task_types import TaskResponse
from meutils.apis.jimeng.common import get_headers, check_token
from meutils.apis.jimeng.files import upload_for_image, upload_for_video
from meutils.apis.volcengine_apis import videos as volc_videos

from meutils.config_utils.lark_utils import get_next_token_for_polling

from fake_useragent import UserAgent

ua = UserAgent()

FEISHU_URL = "https://xchatllm.feishu.cn/sheets/GYCHsvI4qhnDPNtI4VPcdw2knEd?sheet=gAUw8s"  # 视频


async def get_task(task_id: str, token: str = "916fed81175f5186a2c05375699ea40d"):
    if task_id.startswith("cgt-"):
        response = await volc_videos.get_task(task_id)  # content
        video_url = response.get("content", {}).get("video_url")
        data = {"video": video_url}

        # 转为逆向接口
        response = TaskResponse(task_id=task_id, status=response.get("status"), data=data)
        return response

    task_ids = task_id.split()

    url = "/mweb/v1/mget_generate_task"
    headers = get_headers(url, token)

    payload = {"task_id_list": task_ids}
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=60) as client:
        response = await client.post(url, json=payload)
        response.raise_for_status()
        data = response.json()
        logger.debug(bjson(data))

        if video_urls := json_path(data, "$..video_url"):  # 角色检测 create_realman_avatar

            task_data = dict(zip(["video"] * len(video_urls), video_urls))
            response = TaskResponse(task_id=task_id, data=task_data, status="success")
            return response

        else:
            response = TaskResponse(task_id=task_id)
            if (
                    (fail_codes := json_path(data, "$..fail_code"))
                    and fail_codes[-1] != "0"
                    and (messages := json_path(data, "$..fail_msg"))
            ):
                response.message = f"{str(messages).lower().replace('success', '')}:{fail_codes}"
                response.status = "fail"
                return response

            if will_cost := json_path(data, "$..will_cost"):
                response.will_cost = will_cost[0]

            if video_urls := json_path(data, "$..[360p,480p,720p].video_url"):
                response.data = [{"video": _} for _ in video_urls]
                response.status = "success"

            response.fail_code = fail_codes and fail_codes[-1]
            return response


async def create_task(request: VideoRequest, token: Optional[str] = None):
    # 采样
    if 1:  # 走即梦
        response = await volc_videos.create_task(request)
        return TaskResponse(task_id=response["id"])

    token = token or await get_next_token_for_polling(FEISHU_URL, check_token)

    url = "/mweb/v1/generate_video"

    headers = get_headers(url, token)

    task_extra = {
        "promptSource": "custom",
        "originSubmitId": str(uuid.uuid4()),
        "isDefaultSeed": 1,
        "originTemplateId": "",
        "imageNameMapping": {},
        "isUseAiGenPrompt": False,
        "batchNumber": 1
    }
    payload = {
        "submit_id": str(uuid.uuid4()),
        "task_extra": json.dumps(task_extra),
        "input": {
            "video_aspect_ratio": request.aspect_ratio,
            "seed": 1751603315,  ##### seed 10位
            "video_gen_inputs": [
                {
                    "prompt": request.prompt,
                    "fps": 24,
                    "duration_ms": request.duration * 1000,
                    "video_mode": 2,
                    "template_id": ""
                }
            ],
            "priority": 0,
            "model_req_key": "dreamina_ic_generate_video_model_vgfm_3.0",  # request.model
        },
        "mode": "workbench",
        "history_option": {},
        "commerce_info": {
            "resource_id": "generate_video",
            "resource_id_type": "str",
            "resource_sub_type": "aigc",
            "benefit_type": "basic_video_operation_vgfm_v_three"
        },
        "client_trace_data": {}
    }

    if request.image_url:
        # image_url = "tos-cn-i-tb4s082cfz/a116c6a9dcbc41b889f9aabdef645456"
        image_url = await upload_for_image(request.image_url, token, biz="video")
        # vid, uri = await upload_for_video(request.image_url, token)
        # logger.debug(f"vid: {vid}, uri: {uri}")
        payload['input'].pop('video_aspect_ratio', None)
        payload['input']['video_gen_inputs'][0]['first_frame_image'] = {
            "width": 1024,
            "height": 1024,
            "image_uri": image_url
        }

    logger.debug(bjson(payload))

    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=60) as client:
        response = await client.post(url, json=payload)
        response.raise_for_status()
        data = response.json()
        logger.debug(bjson(data))

    if task_ids := json_path(data, "$..task.task_id"):
        task_id = task_ids[0]
        return TaskResponse(task_id=task_id, system_fingerprint=token)

    else:
        """
       {
           "ret": "1018",
           "errmsg": "account punish limit ai generate",
           "systime": "1749027488",
           "logid": "202506041658081AB86654C66682A7DE2E",
           "data": null
       }
        """

        raise Exception(data)


if __name__ == '__main__':
    token = None
    token = "d2d142fc877e696484cc2fc521127b36"

    request = VideoRequest(
        model="dreamina_ic_generate_video_model_vgfm_3.0",
        prompt="笑起来",
        image_url="https://oss.ffire.cc/files/kling_watermark.png",  # 图生有问题
    )

    with timer():
        r = arun(create_task(request))
        print(r)

    # arun(get_task(r.task_id))
    # arun(get_task(r.task_id, "d2d142fc877e696484cc2fc521127b36"))
    # task_id = "4620067333122"
    #
    # arun(get_task(task_id, token))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : llm_qa
# @Time         : 2024/4/24 18:11
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

from langchain.chat_models import ChatOpenAI
from langchain.output_parsers import OutputFixingParser
from langchain.output_parsers import PydanticOutputParser


class ExpressRequest(BaseModel):
    express_no: str = Field(description="快递单号")
    phone_number: str = Field(description="手机号")

    def __init__(self, /, **kwargs):
        super().__init__(**kwargs)
        self.express_no = f"{self.express_no}:{self.phone_number[-4:]}"


parser = PydanticOutputParser(pydantic_object=ExpressRequest)

print(parser.get_format_instructions())


# parser = OutputFixingParser.from_llm(
#     parser=parser,
#     # llm=ChatOpenAI(temperature=0),
#     llm=ChatOpenAI(model="kimi"),
# )
#
text = """
快递号
SF1442153365663
手机号是
121215651
"""
#
# result = parser.parse(text)  # 错误被自动修正
# print(text)
# print(result)


print(parser.parse("""{'快递号': SF1442153365663}"""))


import openai

openai.OpenAI


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : kolors
# @Time         : 2025/1/6 16:39
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.io.files_utils import to_url_fal, to_base64
from meutils.schemas.image_types import KolorsRequest, ImagesResponse
from meutils.config_utils.lark_utils import get_next_token_for_polling
from meutils.decorators.retry import retrying
from openai import AsyncOpenAI

FEISHU_URL = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=gg5DNy"


@retrying(title=__name__)
async def generate(request: KolorsRequest, token: Optional[str] = None):
    """kolors-1.0
    controls = {"image": 'xxx'}

    """
    token = token or await get_next_token_for_polling(FEISHU_URL)

    if image := request.controls.get('image'):  # 图生图
        image = await to_base64(image)  # url / base64
        # 1152 * 896
        # request.size = "1024x1024"
        payload = {
            "inputs": request.prompt,
            "image": image,

            "width": 896,
            "height": 896,
            "steps": request.steps or 25,
            "guidance_scale": request.guidance or 7.5,

        }
        headers = {"Authorization": f"Bearer {token}", "X-Failover-Enabled": "true", "X-Package": "1910"}
        async with httpx.AsyncClient(headers=headers, timeout=60) as client:
            response = await client.post(
                url="https://ai.gitee.com/api/serverless/Kolors/text-to-image",
                json=payload,
            )
            response.raise_for_status()

            url = None
            b64_json = response.content
            if request.response_format == "url":
                url = await to_url_fal(b64_json, content_type="image/png")
                b64_json = None
            return ImagesResponse(data=[{"url": url, "b64_json": b64_json}])

        # todo; 不知道为啥报错
        # client = AsyncOpenAI(
        #     base_url="https://ai.gitee.com",
        #     api_key=token,
        #     default_headers={"X-Failover-Enabled": "true", "X-Package": "1910"},
        # )
        # response = await client.post(
        #     "/api/serverless/Kolors/text-to-image",
        #     body=payload,
        #     cast_to=object,
        # )
        # return response

    client = AsyncOpenAI(
        base_url="https://ai.gitee.com/v1",
        api_key=token,
        default_headers={"X-Failover-Enabled": "true", "X-Package": "1910"},
    )

    response = await client.images.generate(
        model="Kolors",
        prompt=request.prompt,

        size=request.size,
        extra_body={
            "steps": request.steps or 25,
            "guidance_scale": request.guidance or 7.5,
        },
    )
    if request.response_format == "url":
        url = await to_url_fal(response.data[0].b64_json, content_type="image/png")
        response.data[0].url = url
        response.data[0].b64_json = None

    return response


if __name__ == '__main__':
    request = KolorsRequest(prompt="一个小女孩举着横幅，上面写着“新年快乐”", size="1024x1024", response_format="url")
    request.controls["image"] = "https://oss.ffire.cc/files/kling_watermark.png"

    arun(generate(request))



# -*- coding: utf-8 -*-

from __future__ import absolute_import

from .base import Type


class Wasm(Type):
    """Implements the Wasm image type matcher."""

    MIME = 'application/wasm'
    EXTENSION = 'wasm'

    def __init__(self):
        super(Wasm, self).__init__(
            mime=Wasm.MIME,
            extension=Wasm.EXTENSION
        )

    def match(self, buf):
        return buf[:8] == bytearray([0x00, 0x61, 0x73, 0x6d,
                                     0x01, 0x00, 0x00, 0x00])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : kling
# @Time         : 2024/11/28 16:18
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from meutils.decorators.retry import retrying
from meutils.async_task import worker, shared_task

from meutils.apis.vidu.vidu_video import create_task, get_task, ViduRequest


@shared_task
async def do_task(**kwargs):
    return create_task(ViduRequest(**kwargs))


@shared_task
@retrying(3)
async def proxy_task(**kwargs):
    pass

    return response



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : minimax
# @Time         : 2025/4/1 12:14
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
import requests
import json

url = "https://api.minimax.chat/v1/text2voice"
api_key = "请输入您的api key"

payload = json.dumps({
    "gender": "female",
    "age": "old",
    "voice_desc": [
        "Kind and friendly",
        "Kind and amiable",
        "Kind hearted",
        "Calm tone"
    ],
    "text": "真正的危险不是计算机开始像人一样思考，而是人开始像计算机一样思考"
})
headers = {
    'Content-Type': 'application/json',
    'Authorization': f'Bearer {api_key}',
}

response = requests.request("POST", url, headers=headers, data=payload)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : utils
# @Time         : 2024/12/25 18:13
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import os

from meutils.pipe import *
from meutils.llm.clients import AsyncOpenAI
from meutils.caches import rcache
from meutils.db.orm import select_first, update_or_insert
from meutils.schemas.db.oneapi_types import OneapiTask, OneapiUser, OneapiToken
from meutils.apis.oneapi.channel import get_channel_keys


@rcache(ttl=90 * 24 * 3600)
async def token2user(api_key: str):
    filter_kwargs = {
        "key": api_key.removeprefix("sk-"),
    }
    # logger.debug(filter_kwargs)
    if _ := await select_first(OneapiToken, filter_kwargs):
        return _.dict()


@rcache(ttl=15)
async def get_user_quota(api_key: Optional[str] = None, user_id: Optional[int] = None):
    assert any([api_key, user_id]), "api_key or user_id must be provided."

    if not user_id:
        if token_object := await token2user(api_key):
            token_object = OneapiToken(**token_object)

            user_id = token_object.user_id

    filter_kwargs = {
        "id": user_id
    }
    if user_object := await select_first(OneapiUser, filter_kwargs):
        return user_object.quota / 500000


async def polling_keys(biz: str, api_key: Optional[str] = None, batch_size: int = 1,
                       channel_id: Optional[int] = None):  # 轮询
    # all
    if channel_id:
        df = await get_channel_keys(channel_id, base_url='https://api.chatfire.cn')
        return df['key_preview'].to_list()  # 渠道所有 keys

    if batch_size > 1:
        tasks = [polling_keys(biz, api_key) for _ in range(batch_size)]
        api_keys = await asyncio.gather(*tasks)
        return api_keys

    # "biz-channel_id" => sk-channel_id

    try:
        client = AsyncOpenAI(
            # base_url="http://0.0.0.0:8000/v1",
            api_key=api_key
        )
        response = await client.audio.speech.create(model=biz, input=biz, voice=biz, extra_query={"biz": biz})
        api_key = response.json().get("api_key")

        # if api_key:
        #     # logger.debug(bjson(response))
        #     logger.debug("获取轮询key成功")

        return api_key
    except Exception as e:
        logger.debug(e)
        return None


if __name__ == '__main__':
    # from faker import Faker

    # with timer():
    #     arun(get_user_quota(os.getenv("OPENAI_API_KEY")))
    # arun(get_user_quota(user_id=1))

    # async def task():
    #     filter_kwargs = dict(
    #         username=f"{shortuuid.random(length=6)}@chatfire.com",
    #     )
    #     return await update_or_insert(OneapiUser, filter_kwargs)
    #
    #
    # async def main():
    #     await asyncio.gather(*[task() for _ in range(5000)])
    #
    #
    # arun(main())

    # arun(get_user_quota("sk-u8QN3zbulUFcCSvI9CIJ87OYsAONEQXGgSyEPyGC0sJhCFFJ"))
    # arun(get_user_quota("sk-x"))
    # arun(token2user("sk-iPNbgHSRkQ9VUb6iAcCa7a4539D74255A6462d29619d65199"))
    # arun(get_user_quota("sk-u8QN3zbulUFcCSvI9CIJ87OYsAONEQXGgSyEPyGC0sJhCFFJ"))
    # arun(polling_key("test"))
    # arun(polling_key('volc'))
    arun(polling_keys('volc', batch_size=1))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : siliconflow
# @Time         : 2025/8/15 20:29
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from meutils.llm.clients import OpenAI

models_mapping = {

}


def get_models_mapping(startswith: str=""):
    client = OpenAI(
        api_key="rc_89a51d02759c2ad3201b3435c1732f21ad183cf32fb7dfcb62a32c727e2b533e",
        base_url="https://api.featherless.ai/v1"
    )

    models = client.models.list().data
    models = {
        m.id.removeprefix("Pro/").split('/', maxsplit=1)[-1].lower(): m.id.removeprefix("Pro/") for m in models
        if any(i not in m.id.lower() for i in {"stable-diffusion", "wan"})
    }
    if startswith:
        models = {k: v for k, v in models.items() if k.startswith(startswith)}
        return models

    return {**models, **models_mapping}


if __name__ == '__main__':
    startswith = ""
    # startswith = "ling"

    data = get_models_mapping(startswith)
    print(bjson(data))
    print(','.join(data))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : images
# @Time         : 2025/10/21 17:22
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 


from meutils.pipe import *
from meutils.caches import rcache
from meutils.llm.clients import AsyncClient
from meutils.decorators.retry import retrying, IgnoredRetryException

from meutils.llm.openai_utils import to_openai_params
from meutils.io.files_utils import to_png, to_url_fal, to_url
from meutils.notice.feishu import send_message_for_images
from meutils.schemas.image_types import ImageRequest, ImagesResponse

base_url = "https://api.vmodel.ai"

"""

curl -X POST https://api.vmodel.ai/api/tasks/v1/create \
-H "Authorization: Bearer XeK1-4Pr11MjHlaliUN7fTsCSitgacf699wCvnBbbrgUKxX2VIcihYRVkGNx2y_mwJkfe8wlbqehsGPoXK1GvA==" \
-H "Content-Type: application/json" \
-d '{
"version": "6bb2912c9d6eb56fd49bc7a384a0cdf588258237b4f13402102b6b6ca93e587a",
"input": {
    "prompt": "Make this a 90s cartoon",
    "input_image": [
            "https://vmodel.ai/data/model/vmodel/nano-banana/nano1.png",
            "https://vmodel.ai/data/model/vmodel/nano-banana/nano2.png"
        ],
    
    "aspect_ratio": "match_input_image",
    "output_format": "jpg",
    "safety_tolerance": 2,
    "disable_safety_checker": false
}
}'


curl https://api.vmodel.ai/api/tasks/v1/get/ddnvhe2k0oxbshplob \
-H "Authorization: Bearer XeK1-4Pr11MjHlaliUN7fTsCSitgacf699wCvnBbbrgUKxX2VIcihYRVkGNx2y_mwJkfe8wlbqehsGPoXK1GvA=="




  curl -X POST https://api.vmodel.ai/api/tasks/v1/create
    -H "Authorization: Bearer $VModel_API_TOKEN"
    -H "Content-Type: application/json"
    -d '{
    "version": "6ee81ffe35d342a8fecaa47854824401caf367808090fa39eb78e781516419f2",
    "input": {
        "prompt": "Make the letters 3D, floating in space on a city street",
        "input_image": "https://vmodel.ai/data/model/vmodel/flux-kontext-max/flux-kontext-max-input.webp",
        "aspect_ratio": "match_input_image",
        "output_format": "png",
        "safety_tolerance": 6,
        "disable_safety_checker": false,
        "seed": 888,
        "prompt_upsampling": true
    }
}'
    
    

  curl -X POST https://api.vmodel.ai/api/tasks/v1/create
    -H "Authorization: Bearer $VModel_API_TOKEN"
    -H "Content-Type: application/json"
    -d '{
    "version": "44b9310748ecdccd1dfa60d68efe35b4a6291453d5edfad417075890d55a208f",
    "input": {
        "prompt": "Make the sheets in the style of the logo. Includes this logo's color. Make the scene natural.",
        "image_input": [
            "https://vmodel.ai/data/model/vmodel/nano-banana/nano1.png",
            "https://vmodel.ai/data/model/vmodel/nano-banana/nano2.png"
        ],
        "output_format": "jpg",
        "disable_safety_checker": false
    }
}'
    
    
  
  curl -X POST https://api.vmodel.ai/api/tasks/v1/create
    -H "Authorization: Bearer $VModel_API_TOKEN"
    -H "Content-Type: application/json"
    -d '{
    "version": "6ee81ffe35d342a8fecaa47854824401caf367808090fa39eb78e781516419f2",
    "input": {
        "prompt": "Make the letters 3D, floating in space on a city street",
        "input_image": "https://vmodel.ai/data/model/vmodel/flux-kontext-max/flux-kontext-max-input.webp",
        "aspect_ratio": "match_input_image",
        "output_format": "jpg",
        "safety_tolerance": 2,
        "disable_safety_checker": false
    }
}'
    
    
    
"""


async def generate(request: ImageRequest, api_key: Optional[str] = None, base_url: Optional[str] = None):
    task_id = await create_task(request, api_key)

    for i in tqdm(range(20)):
        if data := await get_task(task_id, api_key):
            return data

        await asyncio.sleep(3)


@retrying(ignored_exception_types=IgnoredRetryException)
async def get_task(task_id, api_key):
    client = AsyncClient(api_key=api_key, base_url=base_url)

    response = await client.get(f"/api/tasks/v1/get/{task_id}", cast_to=object)

    logger.debug(bjson(response))

    if error := response.get("result", {}).get("error"):
        raise IgnoredRetryException(error)

    if images := response.get("result", {}).get("output"):
        return ImagesResponse(image=images, **response.get("result"))

    """
    {
    "code": 200,
    "result": {
        "task_id": "ddnxnz9cyhyr67vpd4",
        "user_id": 10061,
        "version": "44b9310748ecdccd1dfa60d68efe35b4a6291453d5edfad417075890d55a208f",
        "error": "The input or output was flagged as sensitive. Please try again with different inputs. (E005) (uIJ6l3ruRD)", #####
        "total_time": 5.0,
        "predict_time": null,
        "logs": null,
        "output": [],
        "status": "failed",
        "create_at": null,
        "completed_at": null
    },
    "message": {}
}



    """


@rcache(ttl=3600)
async def create_task(request: ImageRequest, api_key: Optional[str] = None):
    payload = {
        "version": request.model,
        "input": {
            "prompt": request.prompt,
            "aspect_ratio": request.aspect_ratio or "match_input_image",

            "output_format": "png",
            "safety_tolerance": 6,
            "disable_safety_checker": True,
            "seed": request.seed,
            "prompt_upsampling": True
        }
    }

    if request.image_urls and request.model in {
        "6bb2912c9d6eb56fd49bc7a384a0cdf588258237b4f13402102b6b6ca93e587a",
        "6ee81ffe35d342a8fecaa47854824401caf367808090fa39eb78e781516419f2"
    }:
        payload["input"]["input_image"] = request.image_urls[0]


    else:
        payload["input"]["image_input"] = request.image_urls

    client = AsyncClient(api_key=api_key, base_url=base_url)

    response = await client.post(
        "/api/tasks/v1/create",
        body=payload,
        cast_to=object
    )

    logger.debug(bjson(response))

    if "Payment Required" in str(response):
        raise IgnoredRetryException("insufficient")

    if task_id := response.get("result", {}).get("task_id"):
        """
        {'code': 200,
 'message': {'en': 'Task created successfully'},
 'result': {'task_cost': 4000, 'task_id': 'ddnx54u1uqwetge0zy'}}
        
        """
        return task_id

    raise Exception(f"Create Task Failed: {response}")


if __name__ == '__main__':
    api_key = "cmJVYe4mNVLD3QJUTJ-mLq1rm52qgoOw_P_iB96Rjh6_uBFqjUD1RCVIKEgdMoQ_6gFnz66HS-uIa7lVZKA2MQ=="
    model = "44b9310748ecdccd1dfa60d68efe35b4a6291453d5edfad417075890d55a208f"
    model = "e2b7b93da2dcc13bb51c48fb8372667ad4c1cb18a7bca18f5b5d06a27d2863bb"

    request = ImageRequest(
        model=model,
        # prompt="Make the sheets in the style of the logo. Includes this logo's color. Make the scene natural.",
        # prompt="一个裸体女人",
        prompt='a cat',

        image_urls=[
            "https://vmodel.ai/data/model/vmodel/nano-banana/nano1.png",
            "https://vmodel.ai/data/model/vmodel/nano-banana/nano2.png"
        ],
        # aspect_ratio="match_input_image",
        seed=888,
    )

    # arun(create_task(request, api_key=api_key))
    # arun(get_task("ddnxnz9cyhyr67vpd4"))

    arun(generate(request, api_key=api_key))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : z.py
# @Time         : 2025/8/19 08:32
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : todo bug英文少空格


from meutils.pipe import *
from meutils.caches import rcache
from meutils.db.redis_db import redis_aclient

from openai import AsyncOpenAI
from meutils.llm.openai_utils import to_openai_params, create_chat_completion_chunk

from meutils.schemas.openai_types import CompletionRequest, chat_completion_chunk, chat_completion

from meutils.decorators.retry import retrying
from meutils.config_utils.lark_utils import get_next_token_for_polling

from fake_useragent import UserAgent

ua = UserAgent()

BASE_URL = "https://chat.z.ai/api"
FEISHU_URL = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=x3TJrE"

MODEL="glm-4.7"
class Completions(object):
    def __init__(self, api_key: Optional[str] = None):
        self.api_key = api_key

    async def create(self, request: CompletionRequest):
        token = self.api_key or await get_next_token_for_polling(FEISHU_URL)

        chat_id = str(uuid.uuid4())
        payload = {
            "id": chat_id,
            "chat_id": chat_id,
            "model": request.model,

            "stream": True,

            "params": {},
            "features": {
                "image_generation": False,
                "web_search": False,
                "auto_web_search": False,
                "preview_mode": False,
                "flags": [],
                "features": [
                    {
                        "type": "mcp",
                        "server": "vibe-coding",
                        "status": "hidden"
                    },
                    {
                        "type": "mcp",
                        "server": "ppt-maker",
                        "status": "hidden"
                    },
                    {
                        "type": "mcp",
                        "server": "image-search",
                        "status": "hidden"
                    }
                ],
                "enable_thinking": request.enable_thinking or False
            },

            "background_tasks": {
                "title_generation": False,
                "tags_generation": False
            }
        }

        payload = {**request.model_dump(), **payload}

        data = to_openai_params(payload)

        # todo 代理
        params = {"timestamp": str(int(time.time()*1000)), "signature_timestamp": str(int(time.time()*1000)),}
        default_headers = {
            "X-FE-Version": "prod-fe-1.0.69",
            "X-Signature": "12fd867a781510c83c8a18d81f3a3a304e49c5e6c9cac9f004316a530b1bde1a",
        }

        client = AsyncOpenAI(base_url=BASE_URL, api_key=token, default_headers=default_headers, default_query=params)
        response = await client.chat.completions.create(**data)
        response = self.do_response(response, request.stream)

        # async for i in response:
        #     logger.debug(i)

        return response

    async def do_response(self, response, stream: bool):
        usage = None
        nostream_content = ""
        nostream_reasoning_content = ""
        chat_completion_chunk.model =MODEL
        async for i in response:
            # print(i)

            delta_content = (
                    i.data.get("delta_content", "").removeprefix(
                        '<details type="reasoning" done="false">\n<summary>Thinking…</summary>\n> ')
                    or i.data.get("edit_content", "").split("</details>\n")[-1]
            )

            if i.data.get("phase") == "thinking":
                nostream_reasoning_content += delta_content
                chat_completion_chunk.choices[0].delta.reasoning_content = delta_content

            elif i.data.get("phase") == "answer":
                nostream_content += delta_content
                chat_completion_chunk.choices[0].delta.content = delta_content

            else:
                logger.debug(bjson(i))

            if stream:
                yield chat_completion_chunk

            usage = usage or i.data.get("usage", "")

        if not stream:
            chat_completion.choices[0].message.content = nostream_content
            chat_completion.choices[0].message.reasoning_content = nostream_reasoning_content
            chat_completion.usage = usage
            chat_completion.model = MODEL
            yield chat_completion

    async def upload(self, request: CompletionRequest):  # todo
        files = [
            ('file', (
                'deepinfra.py',
                open('/Users/betterme/PycharmProjects/AI/MeUtils/meutils/apis/images/deepinfra.py', 'rb'),
                'application/octet-stream')
             )
        ]

        headers = {"Authorization": f"Bearer {self.api_key}"}
        async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=120) as client:
            response = await client.post("/v1/files", files=files)
            response.raise_for_status()
            return response.json()


if __name__ == '__main__':
    token = """
    eyJhbGciOiJFUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6IjUxZmNjMWZmLWRlMGYtNDgyYi1hOTAzLTMyMGM0OGJhZTQ2MiIsImVtYWlsIjoiYjZ5bnI5bjlvQG1haWwueGl1dmkuY24ifQ.286d3lCg-p9QUHEBWusC2Oi_a3WgeKtlTcbs1tfXsbW2KatRNxlbtUIeDQBPY5u-Fc3uhoU5ao0DJ4Ww_9dXZw
    """.strip()

    request = CompletionRequest(
        model="GLM-4-6-API-V1",
        messages=[
            {
                "role": "system",
                "content": "你是gpt",

            },
            {
                "role": "user",
                # "content": [{"type": "text", "text": "周杰伦"}],
                # "content": "你是谁",
                "content": "are you ok?",

            }
        ],
        stream=True,

        enable_thinking=True

    )


    async def main():
        response = await Completions(token).create(request)
        async for i in response:
            print(i)


    arun(main())

    # from openai import OpenAI
    #
    # token = "eyJhbGciOiJFUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6ImI0YThkMTI5LWY2YzgtNDM5Mi1iYzlhLWEyNjM1Nzg0ZDM5MyIsImVtYWlsIjoiemJqZ2NlZ2NsbkB0aXRrLnVrIn0.KyRWoGT2vdGN6nPUB5ctU2UkMMtW1XzxVc5KlyWZpIjPqhjHO3gBShFil9j0CG82bdQtA5nDWHIxqzvDcCp-sg"
    #
    # client = OpenAI(base_url="https://chat.z.ai/api/v1/files", api_key=token)

    # response = client.files.create(
    #     file=Path("x.py"),
    #     purpose='vision'
    # )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : video_types
# @Time         : 2024/9/13 10:15
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import json

from meutils.pipe import *
from meutils.math_utils import size2aspect_ratio
from meutils.schemas.utils import to_status

from openai.types.video import Video as _Video


class VideoCreateError(BaseModel):
    code: str = "400"

    message: Optional[str] = None

    def __init__(self, /, **data: Any):
        super().__init__(**data)


class Video(_Video):
    id: str = Field(default_factory=lambda: str(uuid.uuid4()))
    """Unique identifier for the video job."""

    created_at: int = Field(default_factory=lambda: int(time.time()))
    """Unix timestamp (seconds) for when the job was created."""

    model: Optional[Union[str, Literal["sora-2", "sora-2-pro"]]] = None
    """The video generation model that produced the job."""

    object: Literal["image", "video"] = "video"
    """The object type, which is always `video`."""

    progress: int = 0
    """Approximate completion percentage for the generation task."""

    seconds: Optional[Union[str, Literal["4", "8", "12"]]] = None
    """Duration of the generated clip in seconds."""

    size: Optional[Union[str, Literal["720x1280", "1280x720", "1024x1792", "1792x1024"]]] = None
    """The resolution of the generated video."""

    status: Union[Literal["queued", "in_progress", "completed", "failed"], str, dict] = "queued"
    """Current lifecycle status of the video job."""

    video_url: Optional[str] = None  # todo 是否支持多张

    error: Optional[Union[dict, str]] = None

    # "error": {
    #     "code": "generation_failed",
    #     "message": "This content may violate our content policies."
    # },

    # 自定义
    metadata: Optional[Union[dict, str]] = None
    polling_url: Optional[str] = None

    # class Config:
    #     extra = "allow"

    def __init__(self, /, **data: Any):  # todo status映射
        super().__init__(**data)

        self.status = to_status(self.status) or "queued"

        if isinstance(self.error, (tuple, list)):
            self.error = self.error[0]

        if isinstance(self.error, dict):
            self.error["code"] = str(self.error.get("code", "400"))
            self.error["message"] = self.error.get("message", str(self.error))
        else:
            self.error = self.error and {"code": "1", "message": str(self.error)}

        # if self.metadata and not self.video_url: # todo 解析 结果
        #     self.video_url = self.metadata.get("video_url")


class VideoRequest(BaseModel):
    model: Union[str, Literal["cogvideox-flash", "cogvideox"]] = "cogvideox-flash"

    prompt: str = "比得兔开小汽车，游走在马路上，脸上的表情充满开心喜悦。"
    negative_prompt: Optional[str] = None

    """
    提供基于其生成内容的图像。如果传入此参数，系统将以该图像为基础进行操作。支持通过URL或Base64编码传入图片。
    图片要求如下：图片支持.png、jpeg、.jpg 格式、图片大小：不超过5M。image_url和prompt二选一或者同时传入。
    """
    image_url: Optional[str] = None
    tail_image_url: Optional[str] = None

    """
    输出模式，默认为 "quality"。 "quality"：质量优先，生成质量高。 "speed"：速度优先，生成时间更快，质量相对降低。 
    cogvideox-flash模型不支持选择输出模式。
    """
    quality: Literal["quality", "speed"] = "speed"

    """是否生成 AI 音效。默认值: False（不生成音效）。"""
    with_audio: bool = True

    cfg_scale: Optional[float] = None

    """
    默认值: 若不指定，默认生成视频的短边为 1080，长边根据原图片比例缩放。最高支持 4K 分辨率。
    分辨率选项：720x480、1024x1024、1280x960、960x1280、1920x1080、1080x1920、2048x1080、3840x2160
    """
    aspect_ratio: Union[str, Literal["1:1", "21:9", "16:9", "9:16", "4:3", "3:4"]] = "16:9"

    size: Literal[
        '720x480',
        '1024x1024',
        '1280x960',
        '960x1280',
        '1920x1080',
        '1080x1920',
        '2048x1080',
        '3840x2160'] = "1024x1024"

    duration: Literal[5, 10] = 5

    fps: Literal[30, 60] = 30


class SoraVideoRequest(BaseModel):
    model: Union[str, Literal["sora-2", "sora-2-pro"]] = "sora-2"
    prompt: str = "比得兔开小汽车，游走在马路上，脸上的表情充满开心喜悦。"
    seconds: Optional[Union[float, str, Literal["4", "8", "12"]]] = None
    size: Optional[Union[str, Literal["720x1280", "1280x720", "1024x1792", "1792x1024"]]] = None
    input_reference: Optional[Union[str, bytes, list]] = None  # image url/base64/bytes

    # 兼容
    negative_prompt: Optional[str] = None
    duration: Optional[int] = None

    ratio: Optional[str] = None
    aspect_ratio: Optional[str] = None

    resolution: Optional[str] = None

    metadata: Optional[Union[dict, str]] = None

    first_frame_image: Optional[str] = None
    last_frame_image: Optional[str] = None

    #
    image: Optional[str] = None
    audio: Optional[str] = None
    video: Optional[str] = None

    # 声音
    generate_audio: Optional[bool] = None
    enhance_prompt: Optional[bool] = None

    # 万相
    template: Optional[str] = None  # 视频特效模板
    shot_type: Optional[str] = None  # single multi

    # style
    style: Optional[str] = None

    # 回调
    callback_url: Optional[str] = None

    def __init__(self, /, **data: Any):
        super().__init__(**data)
        if isinstance(self.metadata, str):
            self.metadata = json.loads(self.metadata)

        self.duration = self.duration or self.seconds and int(self.seconds)

        if self.input_reference and not isinstance(self.input_reference, list):
            self.input_reference = [self.input_reference]

        if self.size:
            self.size = self.size.replace(':', 'x').replace('*', 'x')
            self.ratio = self.aspect_ratio = size2aspect_ratio(self.size)

        if not self.resolution and '_' in self.model and self.model.lower().endswith(('p', 'auto', 'k', '3840')):
            self.model, self.resolution = self.model.rsplit('_', maxsplit=1)

    class Config:
        extra = "allow"


class FalVideoRequest(BaseModel):
    model: Union[str, Literal["latentsync", "sync-lipsync",]] = 'latentsync'
    video_url: Optional[str] = None
    audio_url: Optional[str] = None
    image_url: Optional[str] = None

    sync_mode: Union[str, Literal["cut_off", "loop", "bounce"]] = "cut_off"


class FalKlingVideoRequest(BaseModel):
    model: Union[
        str, Literal["fal-ai/kling-video/v1/standard/text-to-video",]] = 'fal-ai/kling-video/v1/standard/text-to-video'

    prompt: Optional[str] = None
    duration: Optional[float] = 5.0
    video_url: Optional[str] = None
    audio_url: Optional[str] = None
    image_url: Optional[str] = None

    sync_mode: Union[str, Literal["cut_off", "loop", "bounce"]] = "cut_off"


class LipsyncVideoRequest(BaseModel):
    model: Union[str, Literal[
        "latentsync", "sync-lipsync",
        "lip_sync_avatar_std", "lip_sync_avatar_lively"
    ]
    ] = 'latentsync'

    video_url: Optional[str] = None
    audio_url: Optional[str] = None
    image_url: Optional[str] = None

    sync_mode: Union[str, Literal["cut_off", "loop", "bounce"]] = "cut_off"


if __name__ == '__main__':
    # print(LipsyncVideoRequest())

    # print(Video(x=1))

    print(SoraVideoRequest(
        # model="doubao-seedance-1-0-pro-fast-251015_1080p",
        # model="doubao-seedance-1-0-pro-fast-251015_4k",
        model="doubao-seedance-1-0-pro-fast-251015_3840",

        input_reference="https://example.com/image.jpg",

        size="1x1",
        xx=11111111
    ))

    # data = {
    #     'id': 'cgt-20250613160030-2dvd7',
    #     'model': 'doubao-seedance-1-0-pro-250528',
    #     'status': 'succeeded',
    #     'content': {
    #         'video_url': 'https://ark-content-generation-cn-beijing.tos-cn-beijing.volces.com/doubao-seedance-1-0-pro/02174980163157800000000000000000000ffffac182c17b26890.mp4?X-Tos-Algorithm=TOS4-HMAC-SHA256&X-Tos-Credential=AKLTYjg3ZjNlOGM0YzQyNGE1MmI2MDFiOTM3Y2IwMTY3OTE%2F20250613%2Fcn-beijing%2Ftos%2Frequest&X-Tos-Date=20250613T080120Z&X-Tos-Expires=86400&X-Tos-Signature=5e0928f738f49b93f54923549de4c65940c5007d5e86cb5ebadc756cca3aa03e&X-Tos-SignedHeaders=host'},
    #     'usage': {'completion_tokens': 246840, 'total_tokens': 246840},
    #     'created_at': 1749801631,
    #     'updated_at': 1749801680,
    #     "x": 11111111,
    #     "xx": [1, 2],
    # }
    #
    # _ = Video(
    #     **data
    # )
    #
    # _ = Video(status={
    #     "id": "",
    #     "status": "unknown",
    #     "error": {
    #         "name": "Error",
    #         "message": "invalid params, task_id cannot by empty"
    #     }
    # })
    #
    # print(_)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : vidu_types
# @Time         : 2024/7/31 08:58
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

# BASE_URL = "https://api.vidu.studio/vidu/v1"
BASE_URL = "https://service.vidu.com/vidu/v1"
UPLOAD_BASE_URL = "https://api.vidu.studio/tools/v1"  # /files/uploads

EXAMPLES = [
    {
        "input": {
            "prompts": [
                {
                    "type": "text",
                    "content": "两个人举起茶杯小口抿了一口。左边的人轻抿双唇后微笑，右边的人专注于他们的茶，形成一种静雅和微妙互动的场景。布景精致，淡雅的颜色、花卉布置和古典家具增强了优雅氛围。",
                }
            ],
            "enhance": True,
        },
        "type": "text2video",
        "settings": {
            "style": "general",
            "duration": 4,
            "model": "vidu-1",
            "aspect_ratio": "16:9",

        }
    },
    {
        "input": {
            "prompts": [
                {
                    "type": "text",
                    "content": "开花吧",
                    "enhance": True
                },
                {
                    "type": "image",
                    "content": "ssupload:?id=2368323193735387",
                    "enhance": True
                }
            ]
        },
        "type": "img2video",
        "settings": {
            "style": "general",
            "aspect_ratio": "16:9",
            "duration": 4,
            "model": "vidu-1"
        }
    }
]


# {
#     "input": {
#         "prompts": [
#             {
#                 "type": "text",
#                 "content": "跳起来"
#             },
#             {
#                 "type": "image",
#                 "content": "https://oss.ffire.cc/files/kling_watermark.png",
#                 "src_imgs": [
#                     "https://oss.ffire.cc/files/kling_watermark.png"
#                 ]
#             }
#         ],
#         "enhance": true
#     },
#     "type": "img2video",
#     "settings": {
#         "style": "general",
#         "duration": 4,
#         "model": "vidu-high-performance",
#         "model_version": "1.0"
#     }
# }

class VideoRequest(BaseModel):
    pass


class ViduRequest(BaseModel):
    """quality 倍率2"""
    model: Union[
        str, Literal['vidu-2.0', 'vidu-1.5', 'vidu-high-performance', 'vidu-high-quality']] = "vidu-high-performance"

    prompt: Optional[str] = None

    url: Optional[str] = None  # ssupload:?id=
    tail_image_url: Optional[str] = None

    style: str = "general"  # anime
    aspect_ratio: str = "16:9"
    duration: int = 4

    type: Optional[str] = None  # text2video img2video character2video headtailimg2video

    """vidu-1.5"""
    resolution: Union[str, Literal['512', '720p', '1080p']] = "512"
    movement_amplitude: Optional[str] = "auto"  # small medium high

    sample_count: int = 1
    schedule_mode: Literal["normal", "nopeak"] = "normal"

    payload: dict = {}

    def __init__(self, **data):
        super().__init__(**data)

        if self.duration > 4:
            self.duration = 8
        else:
            self.duration = 4

        input = {
            "prompts": [],
            "enhance": True
        }

        if self.prompt:
            input['prompts'].append(
                {
                    "type": "text",
                    "content": self.prompt,
                }
            )
        type = "text2video"
        if self.url:
            type = "img2video"  # character2video

            input['prompts'].append(
                {
                    "type": "image",
                    "content": self.url,
                    "src_imgs": [self.url, ]
                }
            )

        if self.tail_image_url:
            type = "headtailimg2video"
            input['prompts'].append(
                {
                    "type": "image",
                    "content": self.tail_image_url,
                    "src_imgs": [self.tail_image_url, ]
                }
            )

        if self.model in ("vidu-2.0",):
            self.payload = {
                "input": input,
                "type": self.type or type,
                "settings": {
                    "model_version": "2.0",  #######
                    "style": "general",
                    "duration": self.duration,

                    "resolution": self.resolution,
                    "aspect_ratio": self.aspect_ratio,

                    "movement_amplitude": self.movement_amplitude,
                    "sample_count": self.sample_count,
                    "schedule_mode": self.schedule_mode,

                }
            }

        else:

            self.payload = {
                "input": input,
                "type": self.type or type,
                "settings": {
                    "model": self.model,
                    "model_version": "1.0",

                    "style": self.style,

                    "aspect_ratio": self.aspect_ratio,

                    "duration": self.duration,

                }
            }

    class Config:
        json_schema_extra = {
            "examples": [
                {
                    "prompt": "一只可爱的黑白边境牧羊犬，头伸出车窗，毛发被风吹动，微笑着伸出舌头。",
                }
            ]
        }


class ViduUpscaleRequest(BaseModel):
    task_id: str  # vip
    creation_id: str


# todo: 兼容官方api https://kimi.moonshot.cn/chat/cs33f8j1ch3mt3umbsp0
text2video = {
    "type": "text2video",  # type (string, enum: text2video, img2video, character2video, upscale):
    "model": "vidu-1",
    "style": "general",

    "input": {
        "seed": 123,
        "enhance": True,
        "prompts": [
            {
                "type": "text",
                "content": "小白兔白又白"
            }
        ]
    },

    "output_params": {
        "sample_count": 1,
        "duration": 4
    },
    "moderation": False
}

img2video = {
    "type": "img2video",
    "model": "vidu-1",
    "style": "general",

    "input": {
        "enhance": True,
        "prompts": [
            {
                "type": "text",
                "content": "小白兔白又白"
            },
            {
                "type": "image",
                "content": "https://pic.netbian.com/uploads/allimg/170624/1722311498296151ea67.jpg"
            }
        ]
    },

    "output_params": {
        "sample_count": 1,
        "duration": 4
    },
    "moderation": False
}

i2v_15 = {
    "input": {
        "prompts": [
            {
                "type": "image",
                "content": "ssupload:?id=2608006135309944",
                "src_imgs": [
                    "ssupload:?id=2608006135309944"
                ]
            }
        ],
        "enhance": True
    },
    "type": "img2video",
    "settings": {
        "style": "general",
        "duration": 4,
        "resolution": "512",
        "movement_amplitude": "auto",
        "aspect_ratio": "16:9",
        "sample_count": 1,
        "schedule_mode": "nopeak",
        "model_version": "1.5"
    }
}

i2v = {
    "input": {
        "prompts": [
            {
                "type": "image",
                "content": "ssupload:?id=2608006135309944",
                "src_imgs": [
                    "ssupload:?id=2608006135309944"
                ]
            }
        ],
        "enhance": True
    },
    "type": "img2video",
    "settings": {
        "style": "general",
        "duration": 4,
        "resolution": "512",
        "movement_amplitude": "auto",
        "aspect_ratio": "16:9",
        "sample_count": 1,
        "schedule_mode": "nopeak",
        "model_version": "2.0"
    }
}

headtailimg2video = {
    "input": {
        "prompts": [
            {
                "type": "image",
                "content": "ssupload:?id=2608004743287446",
                "src_imgs": [
                    "ssupload:?id=2607999433445006"
                ],
                "selected_region": {
                    "top_left": {
                        "x": 0,
                        "y": 0
                    },
                    "bottom_right": {
                        "x": 1280,
                        "y": 848
                    }
                }
            },
            {
                "type": "image",
                "content": "ssupload:?id=2608004785735138",
                "src_imgs": [
                    "ssupload:?id=2608004785791540"
                ],
                "selected_region": {
                    "top_left": {
                        "x": 0,
                        "y": 0
                    },
                    "bottom_right": {
                        "x": 1280,
                        "y": 848
                    }
                }
            }
        ],
        "enhance": True
    },
    "type": "headtailimg2video",
    "settings": {
        "style": "general",
        "duration": 4,
        "resolution": "512",
        "movement_amplitude": "auto",
        "aspect_ratio": "16:9",
        "sample_count": 1,
        "schedule_mode": "nopeak",
        "model_version": "2.0"
    }
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : hailuo_types
# @Time         : 2024/10/21 20:16
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

BASE_URL = "https://hailuoai.com"
BASE_URL_ABROAD = "https://hailuoai.video"

FEISHU_URL = "https://xchatllm.feishu.cn/sheets/GYCHsvI4qhnDPNtI4VPcdw2knEd?sheet=eDzlrj"
FEISHU_URL_ABROAD = "https://xchatllm.feishu.cn/sheets/GYCHsvI4qhnDPNtI4VPcdw2knEd?sheet=8rB8xi"

FEISHU_URL_OSS = "https://xchatllm.feishu.cn/sheets/MekfsfVuohfUf1tsWV0cCvTmn3c?sheet=Kcg6QC"

#     """生成中是1 成功是2 失败是5 内容审核7 排队中11"""
# "status": 12,
# "message": "Optimizing prompt in progress...",
status_mapper = {
    11: "Queueing",
    12: "Processing",  # "Optimizing prompt in progress..." # 大于10应该都算正常

    1: "Processing",
    2: "Success",
    5: "Failed",
    7: "Failed",
}


class VideoRequest(BaseModel):
    """https://platform.minimaxi.com/document/video_generation?key=66d1439376e52fcee2853049
    23000 文生视频
    "T2V-01": "23000"
    "I2V-01": "23001"
    "I2V-01-live"： "23011"

    "T2V-01": "23000",
    "I2V-01": "23001",
    "I2V-01-live": "23011",
    "S2V-01": "23021"

    {"desc":"飞起来","useOriginPrompt":false,"fileList":[{"id":"320650134834716679","name":"s.jpg","type":"jpg"}],"modelID":"23001"} # 老模型
    {"desc":"飞起来","useOriginPrompt":false,"fileList":[{"id":"320650134834716679","name":"s.jpg","type":"jpg"}],"modelID":"23011"} # 新模型

    {"desc":"跳动","useOriginPrompt":true,"fileList":[{"id":"304987062153912323","name":"3a71b0bb-3cab-4e69-b1f0-592976d0897b_00001_.png","type":"png"}]}
    """
    model: Union[
        str, Literal["MiniMax-Hailuo-02", "video-01", "T2V-01", "I2V-01", "I2V-01-live", "S2V-01"]] = "video-01"

    """生成视频的描述。(注：需少于2000字)"""
    prompt: Optional[str] = None

    """默认取值为True，模型会自动优化传入的prompt，以提升生成质量。如果需要更精确的控制，可以将此参数设置为False，模型将更加严格地遵循指令。此时建议提供更精细的prompt，以获得最佳效果。"""
    prompt_optimizer: bool = True

    """模型将以此参数中传入的图片为首帧画面来生成视频。支持传入图片的url或base64编码。传入此参数时支持将prompt设置为空字符串或不传入prompt，模型将自主决定画面如何演变。
    传入图片需要满足以下条件：格式为JPG/JPEG/PNG；长宽比大于2:5、小于5:2；短边像素大于300px；体积不大于20MB。"""
    first_frame_image: Optional[str] = None

    """本参数仅当model选择为S2V-01时可用。模型将依据此参数中上传的主体来生成视频。目前仅支持单主体参考（数组长度为 1）。"""
    # subject_reference: list = [{"type": "character", "image": ""}]

    duration: Literal[6, 10] = 6
    resolution: Union[str, Literal["768P", "1080P"]] = "768P"

    callback_url: Optional[str] = None

    "n"
    n: int = 1

    def __init__(self, /, **data: Any):
        super().__init__(**data)


class BaseResponse(BaseModel):
    """
    状态码及其分别含义如下：
    0，请求成功
    1002，触发限流，请稍后再试 1000061 1500009
    1004，账号鉴权失败，请检查 API-Key 是否填写正确
    1008，账号余额不足
    1013，传入参数异常，请检查入参是否按要求填写
    1026，视频描述涉及敏感内容
    1027，生成视频涉及敏感内容

    1500002 Please try again with a different prompt

    status string任务状态，包括以下状态
    Preparing-准备中
    Queueing-队列中
    Processing-生成中
    Success-成功
    Fail-失败
    """
    status_code: int = Field(default=0, alias="code")

    """生成视频任务的状态消息，success 为成功。"""
    status_msg: str = Field(default="success", alias="message")


class Video(BaseModel):
    id: Optional[str] = None
    canAppeal: Optional[bool] = None
    canRetry: Optional[bool] = None
    coverURL: Optional[str] = None
    desc: Optional[str] = None
    message: Optional[str] = None
    originFiles: Optional[List[Any]] = None
    percent: Optional[int] = None
    status: Optional[int] = None
    """生成中是1 成功是2 失败是5 内容审核7 排队中11"""
    videoURL: Optional[str] = None
    downloadURL: Optional[str] = None
    width: Optional[int] = None
    height: Optional[int] = None


class VideoResponse(BaseModel):
    """
        {
        "task_id": "106916112212032",
        "base_resp": {
            "status_code": 0,
            "status_msg": "success"
        }
    }
    {
        "task_id": "176843862716480",
        "status": "Success",
        "file_id": "176844028768320",
        "base_resp": {
            "status_code": 0,
            "status_msg": "success"
        }
    }
    """
    task_id: str
    base_resp: BaseResponse  # response.statusInfo

    """
    SUBMITTED-已提交
    Preparing-准备中
    Queueing-队列中
    Processing-生成中
    Success-成功
    Failed-失败
    """
    status: str = "SUBMITTED"
    file_id: Optional[str] = None  # 通过file_id 可以获取视频地址download_url

    videos: Optional[List[Video]] = None  # response.data.videos

    system_fingerprint: Optional[str] = None

    def __init__(self, /, **data: Any):
        super().__init__(**data)
        if self.file_id is None and self.videos:
            self.file_id = self.videos[0].downloadURL

        if self.videos:
            self.status = status_mapper.get(self.videos[0].status or 2, "Failed")


#
# "data": {
#         "videos": [
#             {
#                 "id": "304988292041920515",
#                 "desc": "跳动",
#                 "coverURL": "",
#                 "videoURL": "",
#                 "status": 1,   # 5 生成失败
#                 "percent": 0,
#                 "message": "正在生成，退出后AI会继续生成",
#                 "canRetry": false,
#                 "width": 0,
#                 "height": 0,
#                 "originFiles": [
#                     {
#                         "id": "304987062153912323",
#                         "url": "https://cdn.hailuoai.com/prod/2024-10-22-12/user/multi_chat_file/b9e76359-07ca-4ef4-9f6d-80d8ab416c9f.png",
#                         "type": "png"
#                     }
#                 ],
#                 "canAppeal": false
#             }
#         ],
#         "processing": true,
#         "cycleTime": 10,
#         "hasMore": false
#     }

"""
refer_voice
请输入您的参考音色ID
refer_instrumental
请输入您的参考伴奏ID
lyrics
##在无垠的星空下\n\n梦开始飞翔\n月光洒在心上\n\n温柔的想象\n在这片宁静中\n\n我们自由歌唱##
model
music-01
audio_setting
{"sample_rate":44100,"bitrate":256000,"format":"mp3"}

"""


class MusicRequet(BaseModel):
    model: str = "music-01"

    lyrics: str

    refer_voice: Optional[str] = None
    refer_instrumental: Optional[str] = None
    audio_setting: dict = {
        "sample_rate": 44100,
        "bitrate": 256000,
        "format": "mp3"
    }


"""
{
   "data":{
      "audio":"hex编码的音频数据",
      "status":2
   },
   "trace_id":"02cb3e6a7f8ada5886fdc11e09452353",
   "base_resp":{
      "status_code":0,
      "status_msg":"success"
   }
}
"""


class MusicResponse(BaseModel):
    trace_id: str
    base_resp: BaseResponse


if __name__ == '__main__':

    print(VideoResponse(task_id='1', base_resp={"status_code": 0, "status_msg": "success"}).model_dump())

    print(Video(x='xxx'))


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : deepx
# @Time         : 2025/3/20 08:53
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
"""
deep + claude

"""

from openai import AsyncOpenAI

from meutils.pipe import *
from meutils.decorators.retry import retrying
from meutils.io.files_utils import to_bytes
from meutils.io.openai_files import file_extract, guess_mime_type
from meutils.str_utils.json_utils import json_path
from meutils.apis.search import metaso

from meutils.llm.clients import chatfire_client, zhipuai_client, AsyncOpenAI
from meutils.llm.openai_utils import to_openai_params

from meutils.schemas.openai_types import ChatCompletionRequest
from meutils.schemas.openai_types import chat_completion, chat_completion_chunk, CompletionRequest, ImageRequest

"""
delta = chunk.choices[0].delta
| │ └ []
| └ ChatCompletionChunk(id='02174299556532927e9140493fc1cd076b4fe1b883ff101a83257', choices=[], created=1742995565, model='deep-d...
|
"""
class Completions(object):

    def __init__(self, api_key: Optional[str] = None):
        self.api_key = api_key
        self.client = AsyncOpenAI(api_key=api_key)

    async def create(self, request: CompletionRequest):
        """
        :param request:
        :return:
        """
        data = to_openai_params(request)
        data['model'] = 'deepseek-reasoner'
        data['max_tokens'] = 1  # 火山 支持max_tokens=1输出思维链
        if request.stream:
            reasoning_content = ""
            completions = await chatfire_client.chat.completions.create(**data)
            async for chunk in completions:
                if chunk.choices:  # 自定义没问题，todo:openai通道报错
                    # logger.debug(chunk)
                    yield chunk
                    delta = chunk.choices[0].delta
                    if hasattr(delta, "reasoning_content"):
                        reasoning_content += delta.reasoning_content
                else:
                    logger.error(chunk)

            request.messages = [
                {
                    'role': 'user',
                    'content': f"""<think>\n\n{reasoning_content}\n\n</think>\n\n{request.last_user_content}"""
                }
            ]
            logger.debug(request)
            data = to_openai_params(request)
            async for chunk in await self.client.chat.completions.create(**data):
                # logger.debug(chunk)
                yield chunk
        else:
            reasoning_content = ""
            completions = await chatfire_client.chat.completions.create(**data)
            message = completions.choices[0].message
            if hasattr(message, "reasoning_content"):
                reasoning_content += message.reasoning_content

            request.messages = [
                {
                    'role': 'user',
                    'content': f"""<think>\n\n{reasoning_content}\n\n</think>\n\n\n{request.last_user_content}"""
                }
            ]
            data = to_openai_params(request)
            _completions = await self.client.chat.completions.create(**data)
            completions.choices[0].message.content = _completions.choices[0].message.content
            yield completions

    async def screate(self, request: CompletionRequest):
        pass


if __name__ == '__main__':
    c = Completions()

    request = CompletionRequest(
        # model="qwen-turbo-2024-11-01",
        # model="claude-3-5-sonnet-20241022",
        model="deepseek-chat",
        stream=True,

        messages=[{
            'role': 'user',
            'content': "1+1"
        }])

    arun(c.create(request))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-App.iapp
# @File         : setup
# @Time         : 2019-06-17 16:12
# @Author       : yuanjie
# @Email        : meutils@qq.com
# @Software     : PyCharm
# @Description  : https://note.qidong.name/2018/01/python-setup-requires/

import time
import pandas as pd
from pathlib import Path
from setuptools import find_packages, setup

# rename
DIR = Path(__file__).resolve().parent
package_name = DIR.name
version = time.strftime("%Y.%m.%d.%H.%M.%S", time.localtime())
Path(DIR / package_name / 'data/VERSION').write_text(version)

get_requirements = lambda p='requirements.txt': pd.read_csv(p, comment='#', names=['name']).name.tolist()
extras_require = {v.name.split('_')[1][:-4]: get_requirements(v) for v in DIR.glob('requirements_*')}
extras_require['all'] = list(set(sum(extras_require.values(), [])))

with open("README.md", encoding='utf-8') as f:
    long_description = f.read()

setup(
    name=package_name,
    version=version,
    url=f'https://github.com/yuanjie-ai/{package_name}',
    keywords=['utils'],
    description='description',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='yuanjie',
    maintainer='yuanjie',
    author_email='313303303@qq.com',
    maintainer_email='313303303@qq.com',
    license='MIT License',
    packages=find_packages(),
    include_package_data=True,
    # package_data={'': ['*.*']},

    platforms=["all"],
    classifiers=[
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python',
        'Programming Language :: Python :: Implementation',
    ],

    python_requires='>=3.6',
    setup_requires=["pandas"],
    install_requires=get_requirements(),
    extras_require=extras_require,  # pip install -U meutils\[all\]

    entry_points={
        'console_scripts': [
            # 通用 'mecli-xx=meutils.clis.xx:cli',
            'mecli=meutils.clis.cli:cli',

            'mecli-cron=meutils.clis.cron:main',
            'mecli-conf=meutils.clis.conf:conf_cli',

            'mecli-monitor=meutils.clis.monitor:cli',

            'mecli-server=meutils.clis.server:cli',

            'mecli-notice=meutils.clis.notice:cli',

            'mecli-browser=meutils.clis.browser:cli',

            'mecli-check-api=meutils.clis.check_api:cli',

            # 定制化
            'nesc=meutils.clis.nesc:cli',

        ]

    },

)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : videos
# @Time         : 2026/1/13 11:34
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import os

from meutils.pipe import *
from openai import AsyncOpenAI
from meutils.schemas.video_types import SoraVideoRequest, Video
# Python 示例代码
import requests
import json


class Tasks(object):

    def __init__(self, base_url: Optional[str] = None, api_key: str = None):
        base_url = "https://api.bizyair.cn/w/v1/webapp/task/openapi"

        self.client = AsyncOpenAI(base_url=base_url, api_key=api_key)

    async def create(self, request: SoraVideoRequest):
        payload = {
            "web_app_id": request.model,
            "suppress_preview_output": True,
            "input_values": {
                # "18:LoadImage.image": "https://bizyair-prod.oss-cn-shanghai.aliyuncs.com/inputs/20251227/CSbjai7F7Cl6CCDZsnzrrU3270kDCmsk.png",
                "54:BizyAir_Sora_V2_I2V_API.prompt": "镜头从杂乱桌面推近到焦虑的二冰侧面，画面分屏展示手动操作步骤快切，最终定格在闪烁的待办便利贴特写。画外音，手动重复操作真折磨人，必须要用自动化搞定！",
                "54:BizyAir_Sora_V2_I2V_API.aspect_ratio": request.ratio,
                "54:BizyAir_Sora_V2_I2V_API.duration": request.seconds,
                "54:BizyAir_Sora_V2_I2V_API.size": "large"
            }
        }



        logany(bjson(payload))

        default_headers = {
            "X-BizyAir-Task-WebHook-Url": f"""{os.getenv("WEBHOOK_URL")}/{payload['model']}"""
        }

        response = await self.client.post(
            path="/video/generations",
            body=payload,
            cast_to=object,
            options={"headers": default_headers}
        )
        """
        生成结果: {'request_id': '8307183e-8387-4e62-895b-a41781a3123c'}

        """

        logger.debug(bjson(response))

        return response

    async def get(self, task_id: str):
        response = await self.client.get(
            path=f"/video/generations?generation_id={task_id}",
            cast_to=object
        )
        """
        {
    "id": "9913239d-4fa8-47ea-b51d-d313e29caba5:alibaba/wan2.5-i2v-preview",
    "status": "completed",
    "video": {
        "url": "https://cdn.aimlapi.com/alpaca/1d/dd/20251107/30b07d9c/42740107-9913239d-4fa8-47ea-b51d-d313e29caba5.mp4?Expires=1762593280&OSSAccessKeyId=LTAI5tBLUzt9WaK89DU8aECd&Signature=Guk6apyEnKeuniLv0mcBJhkHO%2FI%3D"
    }
}

        """
        logger.debug(bjson(response))

        video = Video(
            id=task_id,
            status=response,
            video_url=(response.get("video") or {}).get("url"),

            error=response.get("error")
        )

        # logger.debug(bjson(video))

        return video


api_key = os.getenv("BIZYAIR_API_KEY")

url = "https://api.bizyair.cn/w/v1/webapp/task/openapi/create"
headers = {
    "Content-Type": "application/json",
    "Authorization": f"Bearer {api_key}",
    "X-BizyAir-Task-WebHook-Url": f"""{os.getenv("WEBHOOK_URL")}/sora"""
}
data = {
    "web_app_id": 41928,
    "suppress_preview_output": True,
    "input_values": {
        "18:LoadImage.image": "https://bizyair-prod.oss-cn-shanghai.aliyuncs.com/inputs/20251227/CSbjai7F7Cl6CCDZsnzrrU3270kDCmsk.png",
        "54:BizyAir_Sora_V2_I2V_API.prompt": "镜头从杂乱桌面推近到焦虑的二冰侧面，画面分屏展示手动操作步骤快切，最终定格在闪烁的待办便利贴特写。画外音，手动重复操作真折磨人，必须要用自动化搞定！",
        "54:BizyAir_Sora_V2_I2V_API.aspect_ratio": "9:16",
        "54:BizyAir_Sora_V2_I2V_API.duration": 15,
        "54:BizyAir_Sora_V2_I2V_API.size": "large"
    }
}

response = requests.post(url, headers=headers, json=data)
result = response.json()
print("生成结果:", result)


# X-BizyAir-Task-WebHook-Url

# async def create
# [b'{"type": "API", "status": "Success", "created_at": "2026-01-13 13:09:01", "updated_at": "2026-01-13 13:14:17", "executed_at": "2026-01-13 13:09:02", "running_at": "2026-01-13 13:09:03", "ended_at": "2026-01-13 13:14:17", "expired_at": "2026-01-28 00:00:00", "request_id": "8307183e-8387-4e62-895b-a41781a3123c", "outputs": [{"object_url": "https://storage.bizyair.cn/outputs/e09fca58-56c4-4496-bd1a-97eaea28fe0c_f73b572084eacc48e6284141c9143bb4_video_ComfyUI_91b561c8_00001_.mp4", "output_ext": ".mp4", "cost_time": 315030, "audit_status": 2, "error_type": "NOT_ERROR"}], "cost_times": {"inference_cost_time": 315095, "running_cost_time": 315944, "total_cost_time": 316152, "real_cpu_cost_time": 425, "real_total_cost_time": 4667, "real_bizyair_cost_time": 4242}}']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : zhipu
# @Time         : 2025/2/19 20:42
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from meutils.str_utils.json_utils import json_path
from meutils.llm.clients import zhipuai_sdk_client, zhipuai_client

from meutils.llm.openai_utils import to_openai_params
from meutils.schemas.openai_types import chat_completion, chat_completion_chunk, CompletionRequest, \
    ChatCompletionRequest


class Completions(object):

    def __init__(self, api_key: Optional[str] = None):
        self.api_key = api_key

    async def create(self, request: ChatCompletionRequest, search_result2md: bool = False):
        pass

    # async def _create(self, request: ChatCompletionRequest, search_result2md: bool = False):
    #     request.stream = False
    #     request.model = "web-search-pro"
    #     request.messages = [{
    #         "role": "user",
    #         "content": request.last_content,
    #     }]
    #     data = to_openai_params(request)
    #
    #     search_completion = await zhipuai_client.chat.completions.create(**data)
    #     logger.debug(search_completion.model_dump_json(indent=4))
    #
        # if results := json_path(search_completion, '$..[keywords,query,search_result]'):
    #         data = dict(zip(["keywords", "query", "search_result"], results))
    #         if search_result2md:
    #             global df
    #
    #             df = pd.DataFrame(data["search_result"])
    #
    #             df['title'] = [f"[{k}]({v})" for k, v in zip(df['title'], df['link'])]
    #             df['media'] = [f"![{k}]({v})" for k, v in zip(df['media'], df['icon'])]
    #
    #             df = df[['title', 'media']]
    #             df.index += 1
    #             # {df_.to_markdown(index=False).replace('|:-', '|-').replace('-:|', '-|')}
    #             data["search_result"] = df.to_markdown()
    #         return data

    async def search(self, q: str):
        zhipuai_sdk_client.assistant.conversation(
            model="web-search-pro",
        )

        # {
        #     "role": "user",
        #     "content": search_completion.model_dump_json(indent=4),
        # }


if __name__ == '__main__':
    model = "web-search-pro"
    # model = "tencent-search"

    request = ChatCompletionRequest(
        # model="baichuan4-turbo",
        # model="xx",
        # model="deepseek-r1",
        # model="deepseek-r1:1.5b",
        model=model,

        # model="moonshot-v1-8k",
        # model="doubao",

        messages=[
            {"role": "user", "content": "《哪吒之魔童闹海》现在的票房是多少"}
        ],

        stream=True
    )
    arun(Completions().create(request, search_result2md=True))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : common
# @Time         : 2021/9/10 上午10:45
# @Author       : yuanjie
# @WeChat       : 313303303
# @Software     : PyCharm
# @Description  :
import os
import sys
import time
import wrapt
import asyncio
import inspect
import schedule
import threading
import traceback
import importlib

from loguru import logger
from tqdm.auto import tqdm
from functools import wraps
from concurrent.futures import ThreadPoolExecutor, ProcessPoolExecutor
from contextlib import contextmanager, asynccontextmanager
from typing import *
# ME
from meutils.decorators.decorator import decorator


@decorator
def clear_cuda_cache(func, device='cuda', bins=1, *args, **kwargs):  # todo: 后端运行
    """

    :param device:
    :param bins: 每次都清，bins=2表示每隔一秒（每两秒）一清
    :param args:
    :param kwargs:
    :return:
    """
    if int(time.time()) % bins == 0:
        import torch
        if torch.cuda.is_available():
            with torch.cuda.device(device):  # torch.cuda.current_device()
                torch.cuda.empty_cache()
                torch.cuda.ipc_collect()

        elif torch.backends.mps.is_available():
            try:
                from torch.mps import empty_cache
                empty_cache()
            except Exception as e:
                logger.warning(f"仅支持pytorch2.x: {e}")
    return func(*args, **kwargs)


@decorator
def return2log(func, sink=sys.stderr, logkwargs=None, *args, **kwargs):
    """
        from asgiref.sync import sync_to_async

        @sync_to_async
        def sink(m):
            time.sleep(3)
            print(m)

        @return2log(sink=sink)
        def f(x):
            return x
    """
    if logkwargs is None:
        logkwargs = {}
    logger.remove()
    logger.add(sink, enqueue=True, **logkwargs)
    _ = func(*args, **kwargs)
    logger.info(_)
    return func(*args, **kwargs)


@contextmanager
def timer(task="Task", notice_fn: Optional[Callable] = None):
    """https://www.kaggle.com/lopuhin/mercari-golf-0-3875-cv-in-75-loc-1900-s
        # 其他装饰器可学习这种写法
        with timer() as t:
            time.sleep(3)

        @timer()
        def f():
            print('sleeping')
            time.sleep(6)
            return 6


        from meutils.notice import feishu
        feishu.send_message
    """

    logger.info(f"{task} started")
    s = time.perf_counter()
    yield
    e = time.perf_counter()
    msg = f"{task} done in {e - s:.3f} s"
    logger.info(msg)  # feishu

    # notice
    if notice_fn: notice_fn(msg)


@asynccontextmanager
async def async_timer(task="Task", notice_fn: Optional[Callable] = None):
    """
    Async context manager for timing tasks.

    Usage:
    ```python
    async with async_timer() as t:
        await asyncio.sleep(3)

    @async_timer()
    async def f():
        print('sleeping')
        await asyncio.sleep(6)
        return 6
    ```
    """
    logger.info(f"{task} started")
    s = time.perf_counter()
    yield
    e = time.perf_counter()
    msg = f"{task} done in {e - s:.3f} s"
    logger.info(msg)  # feishu

    # notice
    if notice_fn:
        await notice_fn(msg)


@contextmanager
def tryer(task, is_trace=False):
    try:
        yield
    except Exception as e:
        error = traceback.format_exc() if is_trace else e
        logger.error(error)


@decorator
def do_nothing(func, *args, **kwargs):
    return func(*args, **kwargs)


@decorator
def timeout(func, seconds=1, *args, **kwargs):
    future = ThreadPoolExecutor(1).submit(func, *args, **kwargs)
    return future.result(timeout=seconds)


@decorator
def fork(task, *args, **kwargs):
    """
    def task():
        logger.info(f"task进程：{os.getpid()}")

        for i in range(10) | xtqdm:
            time.sleep(1)
    fork(task)()
    """
    logger.info(f"父进程：{os.getppid()}")

    pid = os.fork()

    if pid < 0:
        logger.error("子进程建立失败")
    elif pid == 0:  # 在子进程中的返回值
        task(*args, **kwargs)
        logger.info(f"{task.__name__} 进程：{os.getpid()}")
    else:  # 在父进程中的返回值
        task(*args, **kwargs)
        logger.info(f"{task.__name__} 进程：{os.getpid()}")


@decorator
def pylock(func, lock=threading.Lock(), *args, **kwargs):
    """https://baijiahao.baidu.com/s?id=1714105650396326932&wfr=spider&for=pc"""
    with lock:
        # lock.acquire()
        _ = func(*args, **kwargs)
        # lock.release()
        return _


@decorator
def timeout(func, seconds=1, *args, **kwargs):
    future = ThreadPoolExecutor(1).submit(func, *args, **kwargs)
    return future.result(timeout=seconds)


@decorator
def background_task(func, max_workers=1, *args, **kwargs):
    # pool.shutdown(wait=False)  # 不等待
    # pool.shutdown(wait=True)  # 等待

    # with ThreadPoolExecutor(max_workers=max_workers, thread_name_prefix='🐶') as pool: # 失去异步效果
    pool = ThreadPoolExecutor(max_workers=max_workers, thread_name_prefix='🐶')
    future = pool.submit(func, *args, **kwargs)  # pool.map(fun4, ips)
    future.add_done_callback(
        lambda x: logger.error(f"future.exception()\n{traceback.format_exc()}") if future.exception() else None
    )
    # future.add_done_callback()

    # pool.shutdown(wait=False)  # 不等待

    return future.running()  # future.done()


background = background_task


@decorator
def background_task_plus(func, *args, **kwargs):
    pool = ThreadPoolExecutor(max_workers=1, thread_name_prefix='🐶')
    future = pool.submit(func, *args, **kwargs)  # pool.map(fun4, ips)
    future.add_done_callback(lambda x: logger.error(f"{future.exception()}") if future.exception() else None)
    # 详细错误 traceback.format_exc()
    return future.running()  # future.done()


# @backend
@decorator
def scheduler(func, scheduler_=schedule.every(2).seconds, stop_func=lambda: False, *args, **kwargs):
    """设置调度的参数，这里是每2秒执行一次

        t = time.time() + 10
        def f():
            time.sleep(1)
            return time.time() > t


        @scheduler(stop_func=f)
        def job(arg):
            print(f"{arg}: a simple scheduler in python.")

        @backend
        @scheduler(stop_func=lambda: False)
        def job():
            global d
            d = {}
            d['t'] = time.ctime() # 后台更新全局变量

    :param func:
    :param scheduler_:
    :param stop_func:
    :param args:
    :param kwargs:
    :return:
    """
    # 先初始化一次
    logger.info(f"{func.__name__} 调度初始化: {func(*args, **kwargs)}")

    # 正式调度
    scheduler_.do(func, *args, **kwargs)

    while True:
        schedule.run_pending()

        if stop_func():
            logger.info(f"{func.__name__} 调度终止")
            break


def add_start_docstrings(*docstr):
    def docstring_decorator(fn):
        fn.__doc__ = "".join(docstr) + (fn.__doc__ if fn.__doc__ is not None else "")
        return fn

    return docstring_decorator


def dependency_exists(dependency):
    try:
        importlib.import_module(dependency)
    except ImportError as e:
        # Check to make sure this isn't some unrelated import error.
        if dependency in repr(e):
            return False
    return True


def requires_dependencies(
        dependencies: Union[str, List[str]],
        extras: Optional[str] = None,
):
    if isinstance(dependencies, str):
        dependencies = [dependencies]

    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            missing_deps = []
            for dep in dependencies:
                if not dependency_exists(dep):
                    missing_deps.append(dep)
            if len(missing_deps) > 0:
                raise ImportError(
                    f"Following dependencies are missing: {', '.join(missing_deps)}. "
                    + (
                        f"""Please install them using `pip install "unstructured[{extras}]"`."""
                        if extras
                        else f"Please install them using `pip install {' '.join(missing_deps)}`."
                    ),
                )
            return func(*args, **kwargs)

        return wrapper

    return decorator


def ratelimit(
        name: str = "ratelimit",
        weight: int = 1,
        rates: Optional[List[Any]] = None,
):
    """ https://github.com/vutran1710/PyrateLimiter
    :param name:
    :param weight:
    :param rates:
    :return:
    """
    from pyrate_limiter import Duration, Rate, Limiter, BucketFullException

    rates = rates or [
        Rate(1, Duration.SECOND),
        Rate(2, Duration.SECOND * 15),
        Rate(2 ** 2, Duration.MINUTE),
        Rate(2 ** 4, Duration.HOUR),
        Rate(2 ** 8, Duration.DAY)
    ]
    limiter = Limiter(rates)

    def name2weight(*args, **kwargs):  # 动态限频 limiter.try_acquire(self, name: str, weight: int = 1)
        return name, weight

    limiter_decorator = limiter.as_decorator()(name2weight)

    @wrapt.decorator
    def wrapper(wrapped, instance, args, kwargs):
        try:
            return limiter_decorator(wrapped)(*args, **kwargs)
        except BucketFullException as e:
            return e.meta_info

    return wrapper


def ratelimiter(limit_value: str = '3/1', callback=None):
    from ratelimiter import RateLimiter

    callback = callback or (lambda x: logger.debug(f"Rate limited: {limit_value}"))

    limiter = RateLimiter(*map(int, limit_value.split('/')), callback=callback)

    @wrapt.decorator
    def inner(wrapped, instance, args, kwargs):
        with limiter:
            return wrapped(*args, **kwargs)

    return inner


def limit(limit_value: str = '1/second', key_func: Callable[..., str] = None, error_message=None):
    """https://github.com/alisaifee/limits"""
    from limits import storage, strategies, parse
    memory_storage = storage.MemoryStorage()
    limiter = strategies.FixedWindowRateLimiter(memory_storage)
    item = parse(limit_value)

    @wrapt.decorator
    def inner(wrapped, instance, args, kwargs):
        # logger.debug(args)
        # logger.debug(kwargs)

        identifier = args[0]
        identifier = key_func and key_func(identifier)  # 根据这个标识限流：比如限制微信用户请求

        if limiter.hit(item, identifier):  # :param identifiers: variable list of strings to uniquely identify this
            return wrapped(*args, **kwargs)
        else:
            return {"error": f"Rate limited: {error_message or item}"}

    return inner


def try_catch(
        task: Optional[str] = None,
        callback: Optional[Callable] = None,
        is_trace: bool = False,
):
    from meutils.notice.feishu import send_message_for_try_catch

    callback = callback or send_message_for_try_catch  # logger.error

    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            task_name = task or f"{func.__module__}.{func.__name__}"

            data = {
                "task_name": task_name,
                # "args": args,
                # "kwargs": kwargs,
                # "defined_args": inspect.getfullargspec(func).args,  # .varargs
            }

            logger.debug(data)

            async def async_wrapper():
                try:
                    if asyncio.iscoroutinefunction(func):
                        return await func(*args, **kwargs)
                    else:
                        return func(*args, **kwargs)
                except Exception as e:

                    if is_trace:
                        error_msg = f"{task_name}:\n{traceback.format_exc()}"
                    else:
                        error_msg = f"{task_name}: {e}"

                    data["error"] = error_msg
                    callback(data)

                    raise

            if asyncio.iscoroutinefunction(func):
                return async_wrapper()
            else:
                try:
                    return asyncio.get_event_loop().run_until_complete(async_wrapper())
                except RuntimeError:
                    # 如果没有事件循环（例如在同步环境中），则直接运行
                    return async_wrapper()

        return wrapper

    return decorator


if __name__ == '__main__':
    import time

    # @timeout()
    # def ff():
    #     import time
    #     time.sleep(30)
    #     return "OK"
    #
    #
    # print(ff())
    #
    #
    # def func():
    #     print(f"开始循环: {time.time()}")
    #     for i in range(10):
    #         time.sleep(1)
    #         print(f"{i}: {time.time()}")
    #
    #
    # @do_more(do_more_func=func)
    # def func_main():
    #     return "hhh"
    #
    #
    # print(func_main())
    # #
    # #
    #
    # def _do_more():
    #     print(f"开始: {time.time()}")
    #     executor = ThreadPoolExecutor(1)
    #     future = executor.submit(func)
    #
    #     # 主逻辑
    #     print(f"结束: {time.time()}")
    #
    #     return "do_more"
    #
    #
    # print(_do_more())

    # @timer()
    # def ff():
    #     import time
    #     time.sleep(3)
    #     return "OK"
    #
    #
    # ff()

    # @ratelimiter()
    # def fn(x):
    #     return x

    # @limit(limit_value='3/second', key_func=lambda x: str(x))
    # def fn(x, y):
    #     return x, y
    #
    #
    # for i in range(5):
    #     print(fn(1, 1))
    #     print(fn(i, 1))

    # @background_task_plus
    # def f():
    #     while 1:
    #         time.sleep(1)
    #         print('####')
    #
    #
    # f()
    # while 1:
    #     pass

    # def f():
    #     return 1 / 0
    #     # return 1
    #
    #
    # with tryer('try'):
    #     a = f()
    # print(a)

    from meutils.pipe import *
    from meutils.notice.feishu import send_message_for_try_catch


    @try_catch(is_trace=False)
    async def my_async_function(x=1, y=1):
        # 您的异步函数代码
        pass
        1 / 0


    async def main():
        await my_async_function(x=111, y=10000)


    # @async_try_catch(task="Custom Task Name", fallback=some_fallback_function)
    # async def another_async_function():
    #     # 另一个异步函数的代码
    #     pass

    arun(main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : cmd
# @Time         : 2021/2/24 2:28 下午
# @Author       : yuanjie
# @WeChat       : 313303303
# @Software     : PyCharm
# @Description  : 


from meutils.pipe import magic_cmd

# build
cmd = 'docker build -f Dockerfile:milvus .'
magic_cmd(cmd)

# push
r = magic_cmd('docker container ls')[1][1]
print(r)
container_id = r.split()[0]

author = 'yuanjie'
ContainerID = container_id
message = 'msg'
ImageName = 'milvus:1.0'

url = 'eijnauy/ten.imoaix.d.rc'[::-1]
cmd = f"docker commit  -a {author} -m {message} {ContainerID} {url}/{ImageName} && docker push {url}/{ImageName}"
print(magic_cmd(cmd, print_output=True)[0])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : chat
# @Time         : 2025/4/23 10:23
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from meutils.schemas.openai_types import CompletionRequest
from meutils.llm.clients import AsyncOpenAI
from meutils.caches import rcache
from meutils.schemas.chatglm_types import VideoRequest, Parameter, VIDEO_BASE_URL, EXAMPLES

from meutils.decorators.retry import retrying
from meutils.notice.feishu import send_message as _send_message
from meutils.config_utils.lark_utils import get_next_token_for_polling
from meutils.db.redis_db import redis_aclient

from fake_useragent import UserAgent

ua = UserAgent()

FEISHU_URL = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=EOZuBW"
BASE_URL = "https://chatglm.cn/chatglm/backend-api"
"https://chatglm.cn/chatglm/backend-api/assistant/stream"


# {
#     "assistant_id": "65940acff94777010aa6b796",
#     "conversation_id": "",
#     "meta_data": {
#         "is_test": false,
#         "input_question_type": "xxxx",
#         "channel": "",
#         "draft_id": "",
#         "is_networking": false,
#         "chat_mode": "",
#         "quote_log_id": "",
#         "platform": "pc"
#     },
#     "messages": [
#         {
#             "role": "user",
#             "content": [
#                 {
#                     "type": "text",
#                     "text": "你是谁"
#                 }
#             ]
#         }
#     ]
# }

# {"id":"68084f7bef14104a4663f6a8","conversation_id":"68084f7bef14104a4663f6a7","assistant_id":"65940acff94777010aa6b796","parts":[],"created_at":"2025-04-23 10:24:59","status":"init","last_error":{},"meta_data":{"input_question_type":"xxxx","if_plus_model":false,"plus_model_available":true,"if_increase_push":false}}

class Completions(object):
    def __init__(self, api_key: Optional[str] = None):
        self.api_key = api_key

    async def create(self, request: CompletionRequest):
        payload = {
            "assistant_id": request.model,
            "conversation_id": "",
            "meta_data": {
                "is_test": False,
                "input_question_type": "xxxx",
                "channel": "",
                "draft_id": "",
                "is_networking": False,
                "chat_mode": "",
                "quote_log_id": "",
                "platform": "pc"
            },
            "messages": request.messages
        }
        # client = AsyncOpenAI(base_url=BASE_URL, api_key=self.get_access_token())
        # response = await client.post(
        #     "/assistant/stream",
        #     cast_to=object,
        #     body=payload,
        #     stream=True,
        # )
        # for i in response:
        #     logger.debug(i)

        url = "https://chatglm.cn/chatglm/backend-api/assistant/stream"
        # response = self.httpx_client.post(url=url, json=payload)
        headers = {
            'Authorization': f"Bearer {self.get_access_token()}",
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36'
        }
        async with httpx.AsyncClient(headers=headers).stream("POST", url=url, json=payload, timeout=200) as response:
            async for chunk in response.aiter_lines():
                logger.debug(chunk)

    @rcache(ttl=1 * 3600)
    async def get_access_token(self, token: Optional[str] = None):
        token = self.api_key or token or await get_next_token_for_polling(FEISHU_URL)

        response = await AsyncOpenAI(base_url=BASE_URL, api_key=token).post(
            "/v1/user/refresh",
            cast_to=object,
        )

        # logger.debug(response)

        return response.get("result", {}).get("accessToken")


if __name__ == '__main__':
    # arun(Completions().get_access_token())

    request = CompletionRequest(
        model="65940acff94777010aa6b796",
        messages=[
            {
                "role": "user",
                "content": [{"type": "text", "text": "你是谁"}],
            }
        ],
        stream=True,
    )

    arun(Completions().create(request))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : openrouter
# @Time         : 2025/8/28 08:59
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

from meutils.llm.clients import OpenAI

models_mapping = {
    "deepseek-v3.1": "deepseek/deepseek-chat-v3.1:free",
    "kimi-k2-250711": "moonshotai/kimi-k2:free",
    "deepseek-r1-250528": "deepseek/deepseek-r1-0528:free",

}


def get_models_mapping():
    client = OpenAI(
        api_key=os.getenv("OPENROUTER_API_KEY"),
        base_url=os.getenv("OPENROUTER_BASE_URL"),
    )

    models = client.models.list().data
    models = {
        m.id.lower().split('/', maxsplit=1)[-1].removesuffix(":free"): m.id
        for m in models
        if m.id.lower().endswith(':free')
    }
    return {**models, **models_mapping}


if __name__ == '__main__':
    data = get_models_mapping()
    print(bjson(data))
    print(','.join(data))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : videos
# @Time         : 2024/10/21 20:17
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://platform.minimaxi.com/document/video_generation?key=66d1439376e52fcee2853049
# https://useapi.net/docs/start-here/setup-minimax
# token 过期时间一个月: 看下free hailuo
# https://jwt.io/

# todo: check token

import oss2

from meutils.pipe import *
from meutils.hash_utils import md5
from meutils.io.files_utils import to_bytes, to_image
from meutils.io.image import image_resize

from meutils.jwt_utils import decode_jwt_token
from meutils.schemas.hailuo_types import BASE_URL_ABROAD as BASE_URL

from meutils.schemas.hailuo_types import VideoRequest, VideoResponse
from meutils.decorators.retry import retrying
from meutils.notice.feishu import send_message as _send_message, VIDEOS

from meutils.apis.hailuoai.yy import get_yy
from meutils.apis.hailuoai.utils import PARAMS as params, get_access_token, upload
from meutils.apis.hailuoai import openai_images

from meutils.schemas.video_types import SoraVideoRequest, Video
from meutils.schemas.image_types import ImageRequest

send_message = partial(
    _send_message,
    title=__name__,
    url=VIDEOS
)

APP_ID = '3001'
VERSION_CODE = '22203'

MODEL_MAPPING = {
    # video-01 video-01 video-01-live2d S2V-01

    "t2v-01": "23000",  # 23010
    "t2v-01-director": "23010",

    "i2v-01": "23001",
    "i2v-01-live": "23011",
    "video-01-live2d": "23011",
    "s2v-01": "23021",

    # "23210" # 要积分
}


class Tasks(object):

    def __init__(self, base_url: Optional[str] = None, api_key: str = None):
        self.api_key = api_key

    async def create(self, request: SoraVideoRequest):
        if request.model.startswith(("nano", "seedream", "gpt-image")):
            image_request = ImageRequest(
                model=request.model,
                prompt=request.prompt,
                size=request.size,
                aspect_ratio=request.aspect_ratio,
                image=request.input_reference,
            )
            logger.debug(image_request)
            return await openai_images.generate(image_request, api_key=self.api_key, is_async=True)
        else:
            return await create_task(request, self.api_key)

    async def get(self, task_id: str):
        logger.debug(task_id)

        return await get_task(task_id, self.api_key)


# @retrying(predicate=lambda r: r.base_resp.status_code in {1000061, 1500009})  # 限流
async def create_task(request: SoraVideoRequest, token: Optional[str] = None):
    refresh_token = token
    token = await get_access_token(refresh_token)

    if request.model.startswith("veo"):
        request.seconds = 8
        if image_urls := request.input_reference:
            request.input_reference = await to_image(image_urls)

    elif request.model.startswith("sora"):
        request.seconds = int(request.seconds or 4)
        request.seconds = min(request.seconds, 12)

        w, h = map(int, request.size.split('x'))
        request.aspect_ratio = "16:9" if w > h else "9:16"

        if image_urls := request.input_reference:
            logger.debug(f"request.size: {request.size}")

            msg = """
            内容违反了Sora指南:
            1. 仅适合18岁以下观众的内容。
            2. 受版权保护的角色和音乐无法生成。
            3. 无法生成真实人物，包括公众人物。
            4. 带有真人面部的图像无法生成。
            """

            _ = await to_image(image_urls[0], response_format="bytes")
            _ = await image_resize(_, request.size, "url")
            request.input_reference = [_]

    if image_urls := request.input_reference or request.first_frame_image or request.last_frame_image:
        request.model = request.model.replace('t2v', 'i2v')

        if request.model.startswith('232'):  # 2.x 首帧 首尾帧   文生统一 23204
            if request.last_frame_image or len(image_urls) == 2:
                request.model = "23210"  # 首尾帧 2.0

            elif request.first_frame_image or len(image_urls) == 1:
                request.model = "23218"  # 首帧 2.3-fast

        elif request.model.startswith('230'):  # 1.0 文生 23000
            request.seconds = 6
            if image_urls:
                request.model = "23001"



    else:
        request.model = request.model.replace('i2v', 't2v')

    payload = {
        "quantity": 1,
        "parameter": {
            "modelID": request.model,
            "desc": request.prompt,
            "fileList": [],
            "useOriginPrompt": False if request.enhance_prompt else True,
            "duration": int(request.seconds),
            "resolution": (request.resolution or "720").rstrip('p').rstrip('P'),
            "aspectRatio": ""
        },
        # "videoExtra": {
        #     "promptStruct": "{\"value\":[{\"type\":\"paragraph\",\"children\":[{\"text\":\"a cat\"}]}],\"length\":5,\"plainLength\":5,\"rawLength\":5}"
        # }
    }

    if request.aspect_ratio:
        payload['parameter']['aspectRatio'] = request.aspect_ratio

    # 图片比例过小，请上传1280x720或720x1280以上规格图片 todo
    if images := request.input_reference:  # todo 处理 file

        payload['parameter']['fileList'] = [
            {
                "frameType": i,
                "url": url,
                # "type": "jpeg",
            }
            for i, url in enumerate(images)
        ]

    if url := request.first_frame_image:
        url = await to_image(url)

        payload['parameter']['fileList'] += [{"url": url, "frameType": 0}]

    if url := request.last_frame_image:
        url = await to_image(url)

        payload['parameter']['fileList'] += [{"url": url, "frameType": 1}]

    logger.debug(bjson(payload))

    path = "/v2/api/multimodal/generate/video"

    headers = {
        'Content-Type': 'application/json',
        'token': token,
        'yy': get_yy(payload, params, path),
    }

    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=60) as client:
        response = await client.post(path, params=params, content=json.dumps(payload))
        response.raise_for_status()

        data = response.json()

        logger.debug(bjson(data))

        if task_id := data.get("data", {}).get("id"):
            return Video(id=task_id, status=data)
        else:
            return Video(status=data, error=data.get("statusInfo"))


async def get_task(task_id: str, token: str):
    payload = {
        "batchInfoList": [
            {
                "batchID": task_id,
                "batchType": 0  # video
            },
            {
                "batchID": task_id,
                "batchType": 1  # image
            }
        ],
        "type": 1
    }

    # {"batchInfoList": [{"batchID": "472359176514654211", "batchType": 0}], "type": 1}

    path = "/v4/api/multimodal/video/processing"
    headers = {
        'Content-Type': 'application/json',
        'token': token,
        'yy': get_yy(payload, params, url=path),
    }

    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=60) as client:
        response = await client.post(path, params=params, content=json.dumps(payload)
                                     )
        response.raise_for_status()
        data = response.json()

        logger.debug(bjson(data))

        # if any(i in str(data) for i in {"内容生成失败", "请求异常"}):
        #     raise Exception(f"invalid image: {task_id} \n\n {request.prompt[:1000]}")

        if batchs := data['data']['batchVideos']:
            for batch in batchs:
                if batch["batchID"] == task_id:
                    _ = batch["assets"][0]
                    logger.debug(bjson(_))

                    # 1: "Processing",
                    # 2: "Success",
                    error = None
                    status = "queued"

                    if _.get("status") == 12:
                        status = "queued"
                    elif _.get("status") == 1:
                        status = "in_progress"
                    elif _.get("status") == 2:
                        status = "completed"
                    elif _.get("status") in {3, 5, 7}:  # 失败枚举值
                        status = "failed"
                        error = {"code": _.get("status"), "message": f"""{_.get("message", "")}\n{_.get("desc")}"""}

                    video = Video(
                        id=task_id,
                        video_url=_.get("downloadURL"),

                        progress=_.get("percent", 11),

                        created_at=_.get("createTime"),

                        status=status,
                        error=error,
                        # metadata=error and _ or None
                    )

                    if video.video_url:
                        video.progress = 100

                    return video
        return Video(id=task_id)


if __name__ == '__main__':
    token = None
    token = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3NzIzNTA5NjUsInVzZXIiOnsiaWQiOiI0Njk4ODIxOTY3NDM1Mjg0NDkiLCJuYW1lIjoiYWZzbCBkcnF2IiwiYXZhdGFyIjoiIiwiZGV2aWNlSUQiOiIzMTE2NzAxODUwMDg0ODg0NTIiLCJpc0Fub255bW91cyI6ZmFsc2V9fQ.3pO0O36-um2fQs0ML0eHwpi0D7rV5yjmnjcpuiZcNKw"

    model = "veo3.1-t2v-fast"
    model = "23000"
    # model = "23218"
    #
    # model = "veo3.1-i2v-fast_1080p"
    # model = "veo3.1-i2v-fast_3840"
    # model = "sora2-i2v"

    # 23010
    # 472553170574090241

    MODEL_MAPPING = {
        # video-01 video-01 video-01-live2d S2V-01

        "t2v-01": "23000",  # 23010
        "i2v-01": "23001",

        "01-director": "23102",
        "01-live": "23011",

        "s2v-01": "23021",

        # 23200 # 2.0 文生
        # 23210 # 2.0

        # 23204 # 2.3 文生
        # 23217 # 2.3

        # 23218 # 2.3-fast
    }

    request = SoraVideoRequest(
        # model="t2v-01",
        model=model,
        # model="S2V-01-live",
        seconds=61,

        # prompt="smile",  # 307145017365086216
        # prompt="把小黄鸭放在衣服上",  # 307145017365086216
        # prompt="一个裸体少女",  # 307145017365086216
        prompt="cat",  # 307145017365086216

        # enhance_prompt=True,

        # input_reference="https://cdn.hailuoai.video/moss/prod/2026-01-26-18/user/multi_chat_file/8ac50491-63ec-46fc-9f98-94d287552003.jpeg"

        # input_reference=["https://s3.ffire.cc/files/jimeng.jpg"] * 2,
        # first_frame_image="https://s3.ffire.cc/files/jimeng.jpg",
        # last_frame_image="https://cdn.hailuoai.video/moss/prod/2026-01-26-18/user/multi_chat_file/8ac50491-63ec-46fc-9f98-94d287552003.jpeg",

        # first_frame_image="https://v3.fal.media/files/penguin/XoW0qavfF-ahg-jX4BMyL_image.webp",
        # last_frame_image="https://v3.fal.media/files/tiger/bml6YA7DWJXOigadvxk75_image.webp",

        # resolution="768P"

        # size="16:9"
        size="720x1280"
    )

    r = arun(create_task(request, token=token))

    # task_id = "hailuoai-469852272096808964"

    # {"quantity": 1, "parameter": {"modelID": "sora2-i2v", "desc": "笑起来啊", "fileList": [{"id": "469914687591321600",
    #                                                                                         "url": "https://cdn.hailuoai.video/moss/prod/2026-01-20-15/user/multi_chat_file/997398be-fe17-4af1-ac9f-7057a54e5587.jpeg?x-oss-process=image/resize,p_50/format,webp",
    #                                                                                         "name": "cropped_1768892531810.jpeg",
    #                                                                                         "type": "jpeg",
    #                                                                                         "frameType": 0}],
    #                               "useOriginPrompt": true, "resolution": "720", "duration": 4, "aspectRatio": "16:9"},
    #  "videoExtra": {
    #      "promptStruct": "{\"value\":[{\"type\":\"paragraph\",\"children\":[{\"text\":\"笑起来啊\"}]}],\"length\":4,\"plainLength\":4,\"rawLength\":4}"}}

    # data = {
    #     "model": "video-01",
    #     "prompt": "画面中两个人非常缓慢地拥抱在一起",
    #     "prompt_optimizer": True,
    #     # "first_frame_image": "https://hg-face-domestic-hz.oss-cn-hangzhou.aliyuncs.com/avatarapp/ai-cache/54883340-954c-11ef-8920-db8e7bfa3fdf.jpeg"
    # }
    # request = VideoRequest(**data)
    # task_id = "472543767246442500"
    # task_id = "472553541371510788"
    # task_id = "472555135056019456"
    # 472564985043406853

    # task_id = "472565145953669122"
    #
    # task_id = "472565145957863429"
    #
    # "data": {
    #     "id": "472567022007164928",
    #     "task": {
    #         "batchID": "472567022002970627",
    #         "videoIDs": [
    #             "472567022007164928"
    #         ]
    #     },
    #     "isFirstGenerate": false,

    task_id = "472578691764686853"
    task_id = "472578758479253512"
    task_id = "472581277544718343"
    task_id = "472581277544718343"
    # arun(get_task(task_id=task_id, token=token))
